#!/usr/bin/env python
import pytest

import numpy as np
import sklearn.datasets as datasets
import statsmodels.api as sm

import pandas_ml.smaccessors.base as base
import pandas_ml.util.testing as tm


class TestBaseRegressor(tm.TestCase):

    def test_methods(self):
        from sklearn.base import clone
        obj = base.StatsModelsRegressor(sm.OLS, missing='raise')
        cloned = clone(obj)
        self.assertIsInstance(cloned, base.StatsModelsRegressor)
        self.assertTrue(cloned.statsmodel is sm.OLS)

        params = cloned.get_params()
        self.assertEqual(params, {'statsmodel': sm.OLS, 'missing': 'raise'})

        setted = cloned.set_params(statsmodel=sm.GLM, missing='none')
        self.assertIsInstance(setted, base.StatsModelsRegressor)
        self.assertEqual(setted.get_params(), {'statsmodel': sm.GLM, 'missing': 'none'})

    def test_OLS(self):
        diabetes = datasets.load_diabetes()
        estimator = base.StatsModelsRegressor(sm.OLS)
        fitted = estimator.fit(diabetes.data, diabetes.target)
        result = estimator.predict(diabetes.data)

        # estimator.score(diabetes.data, diabetes.target)

        import statsmodels.regression.linear_model as lm
        self.assertIsInstance(fitted, lm.RegressionResultsWrapper)

        fitted2 = sm.OLS(diabetes.target, diabetes.data).fit()
        expected = fitted2.predict(diabetes.data)
        self.assert_numpy_array_almost_equal(result, expected)

    def test_precict(self):
        diabetes = datasets.load_diabetes()
        estimator = base.StatsModelsRegressor(sm.OLS)
        with pytest.raises(ValueError, match='StatsModelsRegressor is not fitted to data'):
            estimator.predict(diabetes.data)

    def test_Regressions(self):
        diabetes = datasets.load_diabetes()
        models = ['OLS', 'GLS', 'WLS', 'GLSAR', 'QuantReg', 'GLM', 'RLM']

        for model in models:
            klass = getattr(sm, model)

            estimator = base.StatsModelsRegressor(klass)
            estimator.fit(diabetes.data, diabetes.target)
            result = estimator.predict(diabetes.data)

            expected = klass(diabetes.target, diabetes.data).fit().predict(diabetes.data)
            self.assert_numpy_array_almost_equal(result, expected)

    def test_GEE(self):
        import statsmodels.genmod.generalized_estimating_equations as geq
        diabetes = datasets.load_diabetes()
        models = ['GEE']            # 'OrdinalGEE', 'NominalGEE']
        data = diabetes.data[:100, :]
        target = diabetes.target[:100]
        groups = np.array([0] * 50 + [1] * 50)
        for model in models:
            klass = getattr(sm, model)

            estimator = base.StatsModelsRegressor(klass, groups=groups)
            fitted = estimator.fit(data, target)
            result = estimator.predict(diabetes.data)
            self.assertIsInstance(fitted, geq.GEEResultsWrapper)

            expected = klass(target, data, groups=groups).fit().predict(diabetes.data)
            self.assert_numpy_array_almost_equal(result, expected)

    def test_MixedLM(self):
        import statsmodels.regression.mixed_linear_model as mlm
        diabetes = datasets.load_diabetes()
        models = ['MixedLM']
        data = diabetes.data[:100, :]
        target = diabetes.target[:100]
        groups = np.array([0] * 50 + [1] * 50)
        for model in models:
            klass = getattr(sm, model)

            estimator = base.StatsModelsRegressor(klass, groups=groups)
            fitted = estimator.fit(data, target)
            # result = estimator.predict(diabetes.data)
            # NotImplementedError
            self.assertIsInstance(fitted, mlm.MixedLMResultsWrapper)

            # expected = klass(target, data, groups=groups).fit().predict(diabetes.data)
            # self.assert_numpy_array_almost_equal(result, expected)

    def test_pipeline(self):
        from sklearn.feature_selection import SelectKBest
        from sklearn.feature_selection import f_regression
        from sklearn.pipeline import Pipeline

        diabetes = datasets.load_diabetes()
        models = ['OLS', 'GLS', 'WLS', 'GLSAR', 'QuantReg', 'GLM', 'RLM']

        for model in models:
            klass = getattr(sm, model)

            selector = SelectKBest(f_regression, k=5)
            estimator = Pipeline([('selector', selector),
                                  ('reg', base.StatsModelsRegressor(klass))])

            estimator.fit(diabetes.data, diabetes.target)
            result = estimator.predict(diabetes.data)

            data = SelectKBest(f_regression, k=5).fit_transform(diabetes.data, diabetes.target)
            expected = klass(diabetes.target, data).fit().predict(data)
            self.assert_numpy_array_almost_equal(result, expected)

    def test_gridsearch(self):
        import sklearn.model_selection as ms
        tuned_parameters = {'statsmodel': [sm.OLS, sm.GLS]}
        diabetes = datasets.load_diabetes()

        cv = ms.GridSearchCV(base.StatsModelsRegressor(sm.OLS), tuned_parameters, cv=5, scoring=None)
        fitted = cv.fit(diabetes.data, diabetes.target)
        self.assertTrue(fitted.best_estimator_.statsmodel is sm.OLS)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.neural_network as nn

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestNeuralNtwork(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.neural_network.BernoulliRBM, nn.BernoulliRBM)
        self.assertIs(df.neural_network.MLPClassifier, nn.MLPClassifier)
        self.assertIs(df.neural_network.MLPRegressor, nn.MLPRegressor)

    @pytest.mark.parametrize("algo", ['BernoulliRBM'])
    def test_RBM(self, algo):
        digits = datasets.load_digits()
        df = pdml.ModelFrame(digits)

        mod1 = getattr(df.neural_network, algo)(random_state=self.random_state)
        mod2 = getattr(nn, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(digits.data, digits.target)

        result = df.transform(mod1)
        expected = mod2.transform(digits.data)
        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)



#!/usr/bin/env python

import pandas as pd

import pandas_ml.skaccessors as skaccessors
import pandas_ml.util as util
from pandas_ml.compat import Appender, cache_readonly
from pandas_ml.core.generic import ModelTransformer, _shared_docs


class ModelSeries(ModelTransformer, pd.Series):
    """
    Wrapper for ``pandas.Series`` to support ``sklearn.preprocessing``
    """

    @property
    def _constructor(self):
        return ModelSeries

    def _call(self, estimator, method_name, *args, **kwargs):
        method = self._check_attr(estimator, method_name)

        # needed for preprocessing
        data = self.values.reshape(-1, 1)
        result = method(data, *args, **kwargs)

        return result

    def _wrap_transform(self, transformed, columns=None):
        """
        Wrapper for transform methods
        """
        if len(transformed.shape) == 2:
            if (util._is_1d_harray(transformed) or util._is_1d_varray(transformed)):
                transformed = transformed.flatten()
            else:
                from pandas_ml.core.frame import ModelFrame
                return ModelFrame(transformed, index=self.index)
        return self._constructor(transformed, index=self.index,
                                 name=self.name)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='transform', returned='returned : transformed result'))
    def transform(self, estimator, *args, **kwargs):
        try:
            transformed = super(ModelSeries, self).transform(estimator, *args, **kwargs)
            return transformed
        except ImportError:
            # raise patsy error
            raise
        except:    # noqa
            return pd.Series.transform(self, estimator, *args, **kwargs)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='preprocessing'))
    def preprocessing(self):
        return self._preprocessing

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='preprocessing'))
    def pp(self):
        return self._preprocessing

    @cache_readonly
    def _preprocessing(self):
        return skaccessors.PreprocessingMethods(self)

    @Appender(pd.Series.to_frame.__doc__)
    def to_frame(self, name=None):
        # ported from pandas 0.16
        from pandas_ml.core.frame import ModelFrame

        if name is None:
            df = ModelFrame(self)
        else:
            df = ModelFrame({name: self})

        return df

    @Appender(pd.core.generic.NDFrame.groupby.__doc__)
    def groupby(self, by=None, axis=0, level=None, as_index=True, sort=True,
                group_keys=True, squeeze=False):
        from pandas_ml.core.groupby import groupby
        if level is None and by is None:
            raise TypeError("You have to supply one of 'by' and 'level'")
        axis = self._get_axis_number(axis)
        return groupby(self, by=by, axis=axis, level=level, as_index=as_index,
                       sort=sort, group_keys=group_keys, squeeze=squeeze)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestSklearnBase(tm.TestCase):

    def test_load_dataset(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)
        self.assertEqual(df.shape, (150, 5))

        msg = "'target' can't be specified for sklearn.datasets"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(iris, target='X')



#!/usr/bin/env python

import sklearn.datasets as datasets
import sklearn.decomposition as decomposition
import sklearn.pipeline as pipeline

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestPipeline(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.pipeline.Pipeline, pipeline.Pipeline)
        self.assertIs(df.pipeline.FeatureUnion, pipeline.FeatureUnion)
        self.assertIs(df.pipeline.make_pipeline, pipeline.make_pipeline)
        self.assertIs(df.pipeline.make_union, pipeline.make_union)

    def test_Pipeline(self):

        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        estimators1 = [('reduce_dim', df.decomposition.PCA()), ('svm', df.svm.SVC())]
        pipe1 = df.pipeline.Pipeline(estimators1)

        estimators2 = [('reduce_dim', decomposition.PCA()), ('svm', df.svm.SVC())]
        pipe2 = pipeline.Pipeline(estimators2)

        df.fit(pipe1)
        pipe2.fit(iris.data, iris.target)

        result = df.predict(pipe1)
        expected = pipe2.predict(iris.data)
        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods, _wrap_data_func


class GaussianProcessMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.gaussian_process``.
    """

    _module_name = 'sklearn.gaussian_process'
    _method_mapper = dict(predict={'GaussianProcess': '_predict'})

    @property
    def correlation_models(self):
        """Property to access ``sklearn.gaussian_process.correlation_models``"""

        module_name = 'sklearn.gaussian_process.correlation_models'
        attrs = ['absolute_exponential', 'squared_exponential',
                 'generalized_exponential', 'pure_nugget',
                 'cubic', 'linear']
        return _AccessorMethods(self._df, module_name=module_name, attrs=attrs)

    @property
    def regression_models(self):
        """Property to access ``sklearn.gaussian_process.regression_models``"""

        return RegressionModelsMethods(self._df)

    @classmethod
    def _predict(cls, df, estimator, *args, **kwargs):
        data = df.data.values
        eval_MSE = kwargs.get('eval_MSE', False)
        if eval_MSE:
            y, MSE = estimator.predict(data, *args, **kwargs)
            if y.ndim == 1:
                y = df._constructor_sliced(y, index=df.index)
                MSE = df._constructor_sliced(MSE, index=df.index)
            else:
                y = df._constructor(y, index=df.index)
                MSE = df._constructor(MSE, index=df.index)
            return y, MSE
        else:
            y = estimator.predict(data, *args, **kwargs)
            if y.ndim == 1:
                y = df._constructor_sliced(y, index=df.index)
            else:
                y = df._constructor(y, index=df.index)
            return y


class RegressionModelsMethods(_AccessorMethods):
    _module_name = 'sklearn.gaussian_process.regression_models'


_regression_methods = ['constant', 'linear', 'quadratic']
_attach_methods(RegressionModelsMethods, _wrap_data_func, _regression_methods)



#!/usr/bin/env python
import pytest

import numpy as np
import sklearn.datasets as datasets
import sklearn.cluster as cluster
import sklearn.preprocessing as pp
import sklearn.metrics as m

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestCluster(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.cluster.AffinityPropagation, cluster.AffinityPropagation)
        self.assertIs(df.cluster.AgglomerativeClustering, cluster.AgglomerativeClustering)
        self.assertIs(df.cluster.Birch, cluster.Birch)
        self.assertIs(df.cluster.DBSCAN, cluster.DBSCAN)
        self.assertIs(df.cluster.FeatureAgglomeration, cluster.FeatureAgglomeration)
        self.assertIs(df.cluster.KMeans, cluster.KMeans)
        self.assertIs(df.cluster.MiniBatchKMeans, cluster.MiniBatchKMeans)
        self.assertIs(df.cluster.MeanShift, cluster.MeanShift)
        self.assertIs(df.cluster.SpectralClustering, cluster.SpectralClustering)

        self.assertIs(df.cluster.bicluster.SpectralBiclustering,
                      cluster.bicluster.SpectralBiclustering)
        self.assertIs(df.cluster.bicluster.SpectralCoclustering,
                      cluster.bicluster.SpectralCoclustering)

    def test_estimate_bandwidth(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.cluster.estimate_bandwidth(random_state=self.random_state)
        expected = cluster.estimate_bandwidth(iris.data, random_state=self.random_state)
        self.assertEqual(result, expected)

    def test_k_means(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.cluster.k_means(3, random_state=self.random_state)
        expected = cluster.k_means(iris.data, 3, random_state=self.random_state)

        self.assertEqual(len(result), 3)
        self.assert_numpy_array_almost_equal(result[0], expected[0])

        self.assertIsInstance(result[1], pdml.ModelSeries)
        tm.assert_index_equal(result[1].index, df.index)
        tm.assert_numpy_array_equal(result[1].values, expected[1])

        self.assertAlmostEqual(result[2], expected[2])

    def test_ward_tree(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.cluster.ward_tree()
        expected = cluster.ward_tree(iris.data)

        self.assertEqual(len(result), 4)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assertEqual(result[1], expected[1])
        self.assertEqual(result[2], expected[2])
        self.assertEqual(result[3], expected[3])

        connectivity = np.ones((len(df), len(df)))
        result = df.cluster.ward_tree(connectivity)
        expected = cluster.ward_tree(iris.data, connectivity)

        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assertEqual(result[1], expected[1])
        self.assertEqual(result[2], expected[2])
        self.assert_numpy_array_almost_equal(result[3], expected[3])

    def test_affinity_propagation(self):
        iris = datasets.load_iris()
        similality = np.cov(iris.data)
        df = pdml.ModelFrame(similality)

        result = df.cluster.affinity_propagation()
        expected = cluster.affinity_propagation(similality)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])

        self.assertIsInstance(result[1], pdml.ModelSeries)
        tm.assert_index_equal(result[1].index, df.index)
        tm.assert_numpy_array_equal(result[1].values, expected[1])

    def test_affinity_propagation_class(self):
        from sklearn.datasets.samples_generator import make_blobs

        centers = [[1, 1], [-1, -1], [1, -1]]
        X, labels_true = make_blobs(n_samples=300, centers=centers,
                                    cluster_std=0.5, random_state=0)

        df = pdml.ModelFrame(data=X, target=labels_true)
        af = df.cluster.AffinityPropagation(preference=-50)
        df.fit(af)

        af2 = cluster.AffinityPropagation(preference=-50).fit(X)

        tm.assert_numpy_array_equal(af.cluster_centers_indices_,
                                    af2.cluster_centers_indices_)
        tm.assert_numpy_array_equal(af.labels_, af2.labels_)

    def test_dbscan(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.cluster.dbscan()
        expected = cluster.dbscan(iris.data)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assertIsInstance(result[1], pdml.ModelSeries)
        tm.assert_index_equal(result[1].index, df.index)
        tm.assert_numpy_array_equal(result[1].values, expected[1])

    def test_mean_shift(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.cluster.mean_shift()
        expected = cluster.mean_shift(iris.data)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assertIsInstance(result[1], pdml.ModelSeries)
        tm.assert_index_equal(result[1].index, df.index)
        tm.assert_numpy_array_equal(result[1].values, expected[1])

    def test_spectral_clustering(self):
        N = 50
        m = np.random.random_integers(1, 200, size=(N, N))
        m = (m + m.T) / 2

        df = pdml.ModelFrame(m)
        result = df.cluster.spectral_clustering(random_state=self.random_state)
        expected = cluster.spectral_clustering(m, random_state=self.random_state)

        self.assertIsInstance(result, pdml.ModelSeries)
        tm.assert_index_equal(result.index, df.index)
        tm.assert_numpy_array_equal(result.values, expected)

    @pytest.mark.parametrize("algo", ['KMeans', 'MiniBatchKMeans'])
    def test_KMeans(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.cluster, algo)(3, random_state=self.random_state)
        mod2 = getattr(cluster, algo)(3, random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_KMeans_scores(self):
        digits = datasets.load_digits()
        df = pdml.ModelFrame(digits)

        scaled = pp.scale(digits.data)
        df.data = df.data.pp.scale()
        self.assert_numpy_array_almost_equal(df.data.values, scaled)

        clf1 = cluster.KMeans(init='k-means++', n_clusters=10,
                              n_init=10, random_state=self.random_state)
        clf2 = df.cluster.KMeans(init='k-means++', n_clusters=10,
                                 n_init=10, random_state=self.random_state)
        clf1.fit(scaled)
        df.fit_predict(clf2)

        expected = m.homogeneity_score(digits.target, clf1.labels_)
        self.assertEqual(df.metrics.homogeneity_score(), expected)

        expected = m.completeness_score(digits.target, clf1.labels_)
        self.assertEqual(df.metrics.completeness_score(), expected)

        expected = m.v_measure_score(digits.target, clf1.labels_)
        self.assertEqual(df.metrics.v_measure_score(), expected)

        expected = m.adjusted_rand_score(digits.target, clf1.labels_)
        self.assertEqual(df.metrics.adjusted_rand_score(), expected)

        expected = m.homogeneity_score(digits.target, clf1.labels_)
        self.assertEqual(df.metrics.homogeneity_score(), expected)

        expected = m.silhouette_score(scaled, clf1.labels_, metric='euclidean',
                                      sample_size=300, random_state=self.random_state)
        result = df.metrics.silhouette_score(metric='euclidean', sample_size=300,
                                             random_state=self.random_state)
        self.assertAlmostEqual(result, expected)

    def test_Classifications(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        models = ['AffinityPropagation', 'MeanShift']
        for model in models:
            mod1 = getattr(df.cluster, model)()
            mod2 = getattr(cluster, model)()

            df.fit(mod1)
            mod2.fit(iris.data)

            result = df.predict(mod1)
            expected = mod2.predict(iris.data)

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

    @pytest.mark.parametrize("algo", ['KMeans', 'MiniBatchKMeans'])
    def test_fit_predict(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.cluster, algo)(3, random_state=self.random_state)
        mod2 = getattr(cluster, algo)(3, random_state=self.random_state)

        result = df.fit_predict(mod1)
        expected = mod2.fit_predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        result = df.score(mod1)
        expected = mod2.score(iris.data)
        self.assert_numpy_array_almost_equal(result, expected)

    @pytest.mark.parametrize("algo", ['SpectralBiclustering',
                                      'SpectralCoclustering'])
    def test_Bicluster(self, algo):
        data, rows, columns = datasets.make_checkerboard(
            shape=(300, 300), n_clusters=5, noise=10,
            shuffle=True, random_state=self.random_state)
        df = pdml.ModelFrame(data)

        mod1 = getattr(df.cluster.bicluster, algo)(3, random_state=self.random_state)
        mod2 = getattr(cluster.bicluster, algo)(3, random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(data)

        self.assert_numpy_array_almost_equal(mod1.biclusters_, mod2.biclusters_)



#!/usr/bin/env python

import sklearn.kernel_ridge as kr

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestKernelRidge(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.kernel_ridge.KernelRidge, kr.KernelRidge)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.feature_selection as fs

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestFeatureSelection(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.feature_selection.GenericUnivariateSelect,
                      fs.GenericUnivariateSelect)
        self.assertIs(df.feature_selection.SelectPercentile,
                      fs.SelectPercentile)
        self.assertIs(df.feature_selection.SelectKBest, fs.SelectKBest)
        self.assertIs(df.feature_selection.SelectFpr, fs.SelectFpr)
        self.assertIs(df.feature_selection.SelectFromModel,
                      fs.SelectFromModel)
        self.assertIs(df.feature_selection.SelectFdr, fs.SelectFdr)
        self.assertIs(df.feature_selection.SelectFwe, fs.SelectFwe)
        self.assertIs(df.feature_selection.RFE, fs.RFE)
        self.assertIs(df.feature_selection.RFECV, fs.RFECV)
        self.assertIs(df.feature_selection.VarianceThreshold,
                      fs.VarianceThreshold)

    def test_chi2(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.feature_selection.chi2()
        expected = fs.chi2(iris.data, iris.target)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])

    def test_f_classif(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.feature_selection.f_classif()
        expected = fs.f_classif(diabetes.data, diabetes.target)

        self.assertEqual(len(result), 2)
        tm.assert_numpy_array_equal(result[0], expected[0])
        tm.assert_numpy_array_equal(result[1], expected[1])

    def test_f_regression(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.feature_selection.f_regression()
        expected = fs.f_regression(diabetes.data, diabetes.target)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])

    @pytest.mark.parametrize("algo", ['SelectPercentile', 'SelectFpr',
                                      'SelectFwe', 'VarianceThreshold'])
    def test_Selection(self, algo):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        mod1 = getattr(df.feature_selection, algo)()
        mod2 = getattr(fs, algo)()

        df.fit(mod1)
        mod2.fit(diabetes.data, diabetes.target)

        result = df.transform(mod1)
        expected = mod2.transform(diabetes.data)
        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)



#!/usr/bin/env python

import sklearn.random_projection as rp

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestRandomProjection(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.random_projection.GaussianRandomProjection, rp.GaussianRandomProjection)
        self.assertIs(df.random_projection.SparseRandomProjection, rp.SparseRandomProjection)
        self.assertIs(df.random_projection.johnson_lindenstrauss_min_dim, rp.johnson_lindenstrauss_min_dim)



#!/usr/bin/python
# -*- coding: utf8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from pandas_ml.confusion_matrix.cm import LabeledConfusionMatrix, ConfusionMatrix  # noqa
from pandas_ml.confusion_matrix.bcm import BinaryConfusionMatrix  # noqa



#!/usr/bin/env python

import pandas as pd

try:
    from sklearn.datasets.base import Bunch
except ImportError:

    class Bunch(object):
        pass


def _maybe_sklearn_data(data, target):
    if isinstance(data, Bunch):
        if target is not None:
            raise ValueError("'target' can't be specified for sklearn.datasets")
        # this should be first
        target = data.target
        # instanciate here to add column name
        columns = getattr(data, 'feature_names', None)
        data = pd.DataFrame(data.data, columns=columns)
        return data, target

    return data, target



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.mixture as mixture

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestMixture(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.mixture.GaussianMixture, mixture.GaussianMixture)
        self.assertIs(df.mixture.BayesianGaussianMixture,
                      mixture.BayesianGaussianMixture)

    @pytest.mark.parametrize("algo", ['GaussianMixture',
                                      'BayesianGaussianMixture'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.mixture, algo)(random_state=self.random_state)
        mod2 = getattr(mixture, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python


from pandas_ml.core.accessor import _AccessorMethods


class ManifoldMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.manifold``.
    """

    _module_name = 'sklearn.manifold'

    def locally_linear_embedding(self, n_neighbors, n_components, *args, **kwargs):
        """
        Call ``sklearn.manifold.locally_linear_embedding`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.locally_linear_embedding
        y, squared_error = func(self._data.values, n_neighbors,
                                n_components, *args, **kwargs)
        y = self._constructor(y, index=self._df.index)
        return y, squared_error

    def spectral_embedding(self, *args, **kwargs):
        """
        Call ``sklearn.manifold.spectral_embedding`` using automatic mapping.

        - ``adjacency``: ``ModelFrame.data``
        """
        func = self._module.spectral_embedding
        data = self._data
        embedding = func(data.values, *args, **kwargs)
        embedding = self._constructor(embedding, index=data.index)
        return embedding



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods, _wrap_data_func


class NeighborsMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.neighbors``.
    """

    _module_name = 'sklearn.neighbors'


_neighbor_methods = ['kneighbors_graph', 'radius_neighbors_graph']
_attach_methods(NeighborsMethods, _wrap_data_func, _neighbor_methods)



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class CrossDecompositionMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.cross_decomposition``.
    """

    _module_name = 'sklearn.cross_decomposition'
    _method_mapper = dict(fit={'PLSCanonical': '_fit', 'CCA': '_fit', 'PLSRegression': '_fit'},
                          transform={'PLSCanonical': '_transform', 'CCA': '_transform'},
                          predict={'PLSRegression': '_predict'})

    @classmethod
    def _fit(cls, df, estimator, *args, **kwargs):
        data = df.data.values
        if df.has_target():
            target = df.target.values
            result = estimator.fit(data, Y=target, *args, **kwargs)
        else:
            # not try to pass target if it doesn't exists
            # to catch ValueError from estimator
            result = estimator.fit(data, *args, **kwargs)
        return result

    @classmethod
    def _transform(cls, df, estimator, *args, **kwargs):
        data = df.data.values
        if df.has_target():
            target = df.target.values
            try:
                result = estimator.transform(data, Y=target, *args, **kwargs)
                result = df._constructor(result[0], target=result[1])
            except TypeError:
                result = estimator.transform(data, *args, **kwargs)
                result = df._constructor(result)
        else:
            # not try to pass target if it doesn't exists
            # to catch ValueError from estimator
            result = estimator.transform(data, *args, **kwargs)
            result = df._constructor(result)
        return result

    @classmethod
    def _predict(cls, df, estimator, *args, **kwargs):
        data = df.data.values
        result = estimator.predict(data, *args, **kwargs)
        result = df._constructor(result)
        return result



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class CovarianceMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.covariance``.
    """

    _module_name = 'sklearn.covariance'

    def empirical_covariance(self, *args, **kwargs):
        """
        Call ``sklearn.covariance.empirical_covariance`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.empirical_covariance
        data = self._data
        covariance = func(data.values, *args, **kwargs)
        covariance = self._constructor(covariance, index=data.columns, columns=data.columns)
        return covariance

    def ledoit_wolf(self, *args, **kwargs):
        """
        Call ``sklearn.covariance.ledoit_wolf`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.ledoit_wolf
        data = self._data
        shrunk_cov, shrinkage = func(data.values, *args, **kwargs)
        shrunk_cov = self._constructor(shrunk_cov, index=data.columns, columns=data.columns)
        return shrunk_cov, shrinkage

    def oas(self, *args, **kwargs):
        """
        Call ``sklearn.covariance.oas`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.oas
        data = self._data
        shrunk_cov, shrinkage = func(data.values, *args, **kwargs)
        shrunk_cov = self._constructor(shrunk_cov, index=data.columns, columns=data.columns)
        return shrunk_cov, shrinkage



#!/usr/bin/env python
import pytest

import matplotlib
matplotlib.use('Agg')

import numpy as np                       # noqa
import sklearn.datasets as datasets      # noqa

import pandas_ml as pdml                 # noqa
import pandas_ml.util.testing as tm      # noqa

try:
    import seaborn as sns                # noqa
except ImportError:
    pass


class SeabornCase(tm.PlottingTestCase):

    def setup_method(self):

        try:
            import matplotlib.pyplot     # noqa
        except ImportError:
            import nose
            # matplotlib.use doesn't work on Travis
            # PYTHON=3.4 PANDAS=0.17.1 SKLEARN=0.16.1
            raise nose.SkipTest()

        self.iris = pdml.ModelFrame(datasets.load_iris())

        self.diabetes = pdml.ModelFrame(datasets.load_diabetes())
        # convert columns to str
        self.diabetes.columns = ['col{0}'.format(c) if isinstance(c, int)
                                 else c for c in self.diabetes.columns]


class TestSeabornAttrs(SeabornCase):

    def test_objectmapper(self):

        df = pdml.ModelFrame([])
        self.assertIs(df.sns.palplot, sns.palplot)
        self.assertIs(df.sns.set, sns.set)
        self.assertIs(df.sns.axes_style, sns.axes_style)
        self.assertIs(df.sns.plotting_context, sns.plotting_context)
        self.assertIs(df.sns.set_context, sns.set_context)
        self.assertIs(df.sns.set_color_codes, sns.set_color_codes)
        self.assertIs(df.sns.reset_defaults, sns.reset_defaults)
        self.assertIs(df.sns.reset_orig, sns.reset_orig)
        self.assertIs(df.sns.set_palette, sns.set_palette)
        self.assertIs(df.sns.color_palette, sns.color_palette)
        self.assertIs(df.sns.husl_palette, sns.husl_palette)
        self.assertIs(df.sns.hls_palette, sns.hls_palette)
        self.assertIs(df.sns.cubehelix_palette, sns.cubehelix_palette)
        self.assertIs(df.sns.dark_palette, sns.dark_palette)
        self.assertIs(df.sns.light_palette, sns.light_palette)
        self.assertIs(df.sns.diverging_palette, sns.diverging_palette)
        self.assertIs(df.sns.blend_palette, sns.blend_palette)
        self.assertIs(df.sns.xkcd_palette, sns.xkcd_palette)
        self.assertIs(df.sns.crayon_palette, sns.crayon_palette)
        self.assertIs(df.sns.mpl_palette, sns.mpl_palette)
        self.assertIs(df.sns.choose_colorbrewer_palette,
                      sns.choose_colorbrewer_palette)
        self.assertIs(df.sns.choose_cubehelix_palette,
                      sns.choose_cubehelix_palette)
        self.assertIs(df.sns.choose_light_palette,
                      sns.choose_light_palette)

        self.assertIs(df.sns.choose_dark_palette, sns.choose_dark_palette)
        self.assertIs(df.sns.choose_diverging_palette,
                      sns.choose_diverging_palette)
        self.assertIs(df.sns.despine, sns.despine)
        self.assertIs(df.sns.desaturate, sns.desaturate)
        self.assertIs(df.sns.saturate, sns.saturate)
        self.assertIs(df.sns.set_hls_values, sns.set_hls_values)
        # self.assertIs(df.sns.ci_to_errsize, sns.ci_to_errsize)
        # self.assertIs(df.sns.axlabel, sns.axlabel)


class TestSeabornDistribution(SeabornCase):

    def test_jointplot(self):
        df = self.iris

        jg = df.sns.jointplot(df.columns[1])
        self.assertIsInstance(jg, sns.JointGrid)
        self.assertEqual(jg.ax_joint.get_xlabel(), df.columns[1])
        self.assertEqual(jg.ax_joint.get_ylabel(), '.target')
        tm.close()

        jg = df.sns.jointplot(df.columns[2], df.columns[3])
        self.assertIsInstance(jg, sns.JointGrid)
        self.assertEqual(jg.ax_joint.get_xlabel(), df.columns[2])
        self.assertEqual(jg.ax_joint.get_ylabel(), df.columns[3])

    def test_pairplot(self):
        df = self.iris

        pg = df.sns.pairplot()
        self._check_axes_shape(pg.axes, axes_num=25,
                               layout=(5, 5), figsize=None)
        for i in range(5):
            self.assertEqual(pg.axes[i][0].get_ylabel(), df.columns[i])
            self.assertEqual(pg.axes[-1][i].get_xlabel(), df.columns[i])
        tm.close()

    def test_distplot(self):
        return          # ToDo: only fails on Travis

        df = self.iris

        ax = df.sns.distplot()
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), '.target')
        tm.close()

        # pass scalar (str)
        ax = df.sns.distplot(df.columns[1])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), df.columns[1])
        tm.close()

        # pass Series
        ax = df.sns.distplot(df[df.columns[2]])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), df.columns[2])

    def test_dist_error(self):
        df = pdml.ModelFrame(np.random.randn(100, 5), columns=list('abcde'))

        msg = "a can't be ommitted when ModelFrame doesn't have target column"
        with pytest.raises(ValueError, match=msg):
            df.sns.distplot()

        df.target = df[['a', 'b']]
        self.assertTrue(df.has_multi_targets())

        msg = "a can't be ommitted when ModelFrame has multiple target columns"
        with pytest.raises(ValueError, match=msg):
            df.sns.distplot()

    def test_kdeplot(self):
        df = pdml.ModelFrame(np.random.randn(100, 5), columns=list('abcde'))
        df.target = df['a']

        ax = df.sns.kdeplot()
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), '')
        self.assertEqual(ax.get_ylabel(), '')
        tm.close()

        ax = df.sns.kdeplot(data='b', data2='c')
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'b')
        self.assertEqual(ax.get_ylabel(), 'c')
        tm.close()

        ax = df.sns.kdeplot(data=df['b'], data2=df['c'])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'b')
        self.assertEqual(ax.get_ylabel(), 'c')

    def test_rugplot(self):
        df = self.iris

        ax = df.sns.rugplot()
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        # rugplot does not add label

        # pass scalar (str)
        ax = df.sns.rugplot(df.columns[1])
        self.assertIsInstance(ax, matplotlib.axes.Axes)

        # pass Series
        ax = df.sns.rugplot(df[df.columns[2]])
        self.assertIsInstance(ax, matplotlib.axes.Axes)

    def test_kde_rug_mix(self):
        import matplotlib.pyplot as plt

        df = pdml.ModelFrame(np.random.randn(100, 5), columns=list('abcde'))
        df.target = df['a']

        f, ax = plt.subplots(figsize=(6, 6))
        ax = df.sns.kdeplot('b', 'c', ax=ax)
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'b')
        self.assertEqual(ax.get_ylabel(), 'c')
        # plot continues, do not reset by tm.close()

        ax = df.sns.rugplot('b', color="g", ax=ax)
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'b')
        self.assertEqual(ax.get_ylabel(), 'c')

        ax = df.sns.rugplot('c', vertical=True, ax=ax)
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'b')
        self.assertEqual(ax.get_ylabel(), 'c')


class TestSeabornRegression(SeabornCase):

    def test_lmplot(self):
        df = self.diabetes

        fg = df.sns.lmplot(df.columns[1])
        self.assertIsInstance(fg, sns.FacetGrid)
        self.assertEqual(fg.ax.get_xlabel(), df.columns[1])
        self.assertEqual(fg.ax.get_ylabel(), '.target')
        tm.close()

        fg = df.sns.lmplot(df.columns[1], df.columns[2])
        self.assertIsInstance(fg, sns.FacetGrid)
        self.assertEqual(fg.ax.get_xlabel(), df.columns[1])
        self.assertEqual(fg.ax.get_ylabel(), df.columns[2])

    def test_regression_plot(self):
        df = self.diabetes

        plots = ['regplot', 'residplot']

        for plot in plots:
            func = getattr(df.sns, plot)

            ax = func(df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), '.target')
            tm.close()

            ax = func(df.columns[1], df.columns[2])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), df.columns[2])
            tm.close()

    def test_interactplot(self):
        pass

    def test_coefplot(self):
        pass


class TestSeabornCategorical(SeabornCase):

    def test_factorplots(self):
        df = self.iris

        fg = df.sns.factorplot(df.columns[1])
        self.assertIsInstance(fg, sns.FacetGrid)
        self.assertEqual(fg.ax.get_xlabel(), '.target')
        self.assertEqual(fg.ax.get_ylabel(), df.columns[1])
        tm.close()

        fg = df.sns.factorplot(x=df.columns[1])
        self.assertIsInstance(fg, sns.FacetGrid)
        self.assertEqual(fg.ax.get_xlabel(), df.columns[1])
        self.assertEqual(fg.ax.get_ylabel(), '.target')
        tm.close()

        fg = df.sns.factorplot(x=df.columns[1], y=df.columns[2])
        self.assertIsInstance(fg, sns.FacetGrid)
        self.assertEqual(fg.ax.get_xlabel(), df.columns[1])
        self.assertEqual(fg.ax.get_ylabel(), df.columns[2])

    def test_categoricalplots(self):
        df = self.iris

        plots = ['boxplot', 'violinplot', 'stripplot']

        for plot in plots:
            func = getattr(df.sns, plot)
            ax = func(df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), '.target')
            self.assertEqual(ax.get_ylabel(), df.columns[1])
            tm.close()

            ax = func(y=df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), '.target')
            self.assertEqual(ax.get_ylabel(), df.columns[1])
            tm.close()

            ax = func(x=df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), '.target')
            tm.close()

            ax = func(x=df.columns[1], y=df.columns[2])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), df.columns[2])
            tm.close()

    def test_categorical_mean_plots(self):
        df = self.iris

        plots = ['pointplot', 'barplot']

        for plot in plots:
            func = getattr(df.sns, plot)
            ax = func(df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), '.target')
            self.assertEqual(ax.get_ylabel(), '{0}'.format(df.columns[1]))
            tm.close()

            ax = func(y=df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), '.target')
            self.assertEqual(ax.get_ylabel(), '{0}'.format(df.columns[1]))
            tm.close()

            ax = func(x=df.columns[1])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), '{0}'.format('.target'))
            tm.close()

            ax = func(x=df.columns[1], y=df.columns[2])
            self.assertIsInstance(ax, matplotlib.axes.Axes)
            self.assertEqual(ax.get_xlabel(), df.columns[1])
            self.assertEqual(ax.get_ylabel(), '{0}'.format(df.columns[2]))
            tm.close()

    def test_count_plots(self):
        df = self.iris

        ax = df.sns.countplot()
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), '.target')
        self.assertEqual(ax.get_ylabel(), 'count')
        tm.close()

        return      # ToDo: only fails on Travis

        ax = df.sns.countplot(df.columns[1])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), df.columns[1])
        self.assertEqual(ax.get_ylabel(), 'count')
        tm.close()

        ax = df.sns.countplot(x=df.columns[1])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), df.columns[1])
        self.assertEqual(ax.get_ylabel(), 'count')
        tm.close()

        ax = df.sns.countplot(y=df.columns[1])
        self.assertIsInstance(ax, matplotlib.axes.Axes)
        self.assertEqual(ax.get_xlabel(), 'count')
        self.assertEqual(ax.get_ylabel(), df.columns[1])
        tm.close()

        with tm.assertRaises(TypeError):
            # can't pass both x and y
            df.sns.countplot(x=df.columns[1], y=df.columns[2])

    # Matrix

    def test_heatmap(self):
        pass

    def test_clustermap(self):
        pass

    # Timeseries

    def test_tsplot(self):
        pass

    # AxisGrid

    def test_facetgrid(self):
        df = self.iris

        fg = df.sns.FacetGrid(df.columns[0])
        self.assertIsInstance(fg, sns.FacetGrid)
        self._check_axes_shape(fg.axes, axes_num=3, layout=(3, 1), figsize=None)
        tm.close()

        fg = df.sns.FacetGrid(row=df.columns[0])
        self.assertIsInstance(fg, sns.FacetGrid)
        self._check_axes_shape(fg.axes, axes_num=3, layout=(3, 1), figsize=None)
        tm.close()

        fg = df.sns.FacetGrid(col=df.columns[0])
        self.assertIsInstance(fg, sns.FacetGrid)
        self._check_axes_shape(fg.axes, axes_num=3, layout=(1, 3), figsize=None)
        tm.close()

    def test_pairgrid(self):
        df = self.iris

        pg = df.sns.PairGrid()
        self.assertIsInstance(pg, sns.PairGrid)
        self._check_axes_shape(pg.axes, axes_num=25, layout=(5, 5), figsize=None)

    def test_jointgrid(self):
        df = self.iris

        jg = df.sns.JointGrid(x=df.columns[1], y=df.columns[1])
        self.assertIsInstance(jg, sns.JointGrid)



#!/usr/bin/env python

from pandas_ml.compat import cache_readonly
from pandas_ml.core.accessor import _AccessorMethods


class ImbalanceMethods(_AccessorMethods):
    """
    Accessor to ``imblearn``.
    """

    _module_name = 'imblearn'

    @property
    def under_sampling(self):
        """Property to access ``imblearn.under_sampling``"""
        return self._under_sampling

    @cache_readonly
    def _under_sampling(self):
        return _AccessorMethods(self._df, module_name='imblearn.under_sampling')

    @property
    def over_sampling(self):
        """Property to access ``imblearn.over_sampling``"""
        return self._over_sampling

    @cache_readonly
    def _over_sampling(self):
        return _AccessorMethods(self._df, module_name='imblearn.over_sampling')

    @property
    def combine(self):
        """Property to access ``imblearn.combine``"""
        return self._combine

    @cache_readonly
    def _combine(self):
        return _AccessorMethods(self._df, module_name='imblearn.combine')

    @property
    def ensemble(self):
        """Property to access ``imblearn.ensemble``"""
        return self._ensemble

    @cache_readonly
    def _ensemble(self):
        return _AccessorMethods(self._df, module_name='imblearn.ensemble')



#!/usr/bin/python
# -*- coding: utf8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import numpy as np
from pandas_ml.confusion_matrix.abstract import ConfusionMatrixAbstract


"""
A Python Pandas Confusion matrix implementation
"""


class ConfusionMatrix(ConfusionMatrixAbstract):
    def __new__(cls, y_true, y_pred, *args, **kwargs):
        uniq_true = np.unique(y_true)
        uniq_pred = np.unique(y_pred)
        if len(uniq_true) <= 2 and len(uniq_pred) <= 2:
            if len(set(uniq_true) - set(uniq_pred)) == 0:
                from pandas_ml.confusion_matrix.bcm import BinaryConfusionMatrix
                return BinaryConfusionMatrix(y_true, y_pred, *args, **kwargs)
        return LabeledConfusionMatrix(y_true, y_pred, *args, **kwargs)


class LabeledConfusionMatrix(ConfusionMatrixAbstract):
    """
    Confusion matrix class (not binary)
    """
    def __getattr__(self, attr):
        """
        Returns (weighted) average statistics
        """
        return(self._avg_stat(attr))



#!/usr/bin/env python

import numpy as np
import sklearn.gaussian_process as gp

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestGaussianProcess(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        dgp = df.gaussian_process

        self.assertIs(dgp.GaussianProcessClassifier,
                      gp.GaussianProcessClassifier)
        self.assertIs(dgp.GaussianProcessRegressor,
                      gp.GaussianProcessRegressor)
        self.assertIs(dgp.correlation_models.absolute_exponential,
                      gp.correlation_models.absolute_exponential)
        self.assertIs(dgp.correlation_models.squared_exponential,
                      gp.correlation_models.squared_exponential)
        self.assertIs(dgp.correlation_models.generalized_exponential,
                      gp.correlation_models.generalized_exponential)
        self.assertIs(dgp.correlation_models.pure_nugget,
                      gp.correlation_models.pure_nugget)
        self.assertIs(dgp.correlation_models.cubic,
                      gp.correlation_models.cubic)
        self.assertIs(dgp.correlation_models.linear,
                      gp.correlation_models.linear)

    def test_objectmapper_abbr(self):
        df = pdml.ModelFrame([])
        dgp = df.gp

        self.assertIs(dgp.GaussianProcessClassifier,
                      gp.GaussianProcessClassifier)
        self.assertIs(dgp.GaussianProcessRegressor,
                      gp.GaussianProcessRegressor)

    def test_objectmapper_kernels(self):
        df = pdml.ModelFrame([])
        dgp = df.gaussian_process

        self.assertIs(dgp.kernels.Kernel, gp.kernels.Kernel)
        self.assertIs(dgp.kernels.Sum, gp.kernels.Sum)
        self.assertIs(dgp.kernels.Product, gp.kernels.Product)
        self.assertIs(dgp.kernels.Exponentiation, gp.kernels.Exponentiation)
        self.assertIs(dgp.kernels.ConstantKernel, gp.kernels.ConstantKernel)
        self.assertIs(dgp.kernels.WhiteKernel, gp.kernels.WhiteKernel)
        self.assertIs(dgp.kernels.RBF, gp.kernels.RBF)
        self.assertIs(dgp.kernels.Matern, gp.kernels.Matern)
        self.assertIs(dgp.kernels.RationalQuadratic, gp.kernels.RationalQuadratic)
        self.assertIs(dgp.kernels.ExpSineSquared, gp.kernels.ExpSineSquared)
        self.assertIs(dgp.kernels.DotProduct, gp.kernels.DotProduct)
        self.assertIs(dgp.kernels.PairwiseKernel, gp.kernels.PairwiseKernel)
        self.assertIs(dgp.kernels.CompoundKernel, gp.kernels.CompoundKernel)
        self.assertIs(dgp.kernels.Hyperparameter, gp.kernels.Hyperparameter)

    def test_constant(self):
        X = np.atleast_2d([1., 3., 5., 6., 7., 8.]).T
        df = pdml.ModelFrame(X)

        result = df.gaussian_process.regression_models.constant()
        expected = gp.regression_models.constant(X)
        self.assert_numpy_array_almost_equal(result, expected)

    def test_linear(self):
        X = np.atleast_2d([1., 3., 5., 6., 7., 8.]).T
        df = pdml.ModelFrame(X)

        result = df.gaussian_process.regression_models.linear()
        expected = gp.regression_models.linear(X)
        self.assert_numpy_array_almost_equal(result, expected)

    def test_quadratic(self):
        X = np.atleast_2d([1., 3., 5., 6., 7., 8.]).T
        df = pdml.ModelFrame(X)

        result = df.gaussian_process.regression_models.quadratic()
        expected = gp.regression_models.quadratic(X)
        self.assert_numpy_array_almost_equal(result, expected)

    def test_GaussianProcess_ge_018(self):
        X = np.atleast_2d([1., 3., 5., 6., 7., 8.]).T
        y = np.sin(X).ravel()
        df = pdml.ModelFrame(X, target=y)

        k1 = (df.gp.kernels.ConstantKernel(1.0, (1e-3, 1e3))
              * df.gp.kernels.RBF(10, (1e-2, 1e2)))
        g1 = df.gp.GaussianProcessRegressor(kernel=k1, n_restarts_optimizer=9,
                                            random_state=self.random_state)

        k2 = (gp.kernels.ConstantKernel(1.0, (1e-3, 1e3))
              * gp.kernels.RBF(10, (1e-2, 1e2)))
        g2 = gp.GaussianProcessRegressor(kernel=k2, n_restarts_optimizer=9,
                                         random_state=self.random_state)

        g1.fit(X, y)
        g2.fit(X, y)

        x = np.atleast_2d(np.linspace(0, 10, 1000)).T
        tdf = pdml.ModelFrame(x)

        y_result = tdf.predict(g1)
        y_expected = g2.predict(x)

        self.assertIsInstance(y_result, pdml.ModelSeries)
        tm.assert_index_equal(y_result.index, tdf.index)

        self.assert_numpy_array_almost_equal(y_result, y_expected)

    def test_GaussianProcess_std(self):
        X = np.atleast_2d([1., 3., 5., 6., 7., 8.]).T
        y = np.sin(X).ravel()
        df = pdml.ModelFrame(X, target=y)

        k1 = (df.gp.kernels.ConstantKernel(1.0, (1e-3, 1e3))
              * df.gp.kernels.RBF(10, (1e-2, 1e2)))
        g1 = df.gp.GaussianProcessRegressor(kernel=k1, n_restarts_optimizer=9,
                                            random_state=self.random_state)

        k2 = (gp.kernels.ConstantKernel(1.0, (1e-3, 1e3))
              * gp.kernels.RBF(10, (1e-2, 1e2)))
        g2 = gp.GaussianProcessRegressor(kernel=k2, n_restarts_optimizer=9,
                                         random_state=self.random_state)

        g1.fit(X, y)
        g2.fit(X, y)

        x = np.atleast_2d(np.linspace(0, 10, 1000)).T
        tdf = pdml.ModelFrame(x)

        y_result, std_result = tdf.predict(g1, return_std=True)
        y_expected, std_expected = g2.predict(x, return_std=True)

        self.assertIsInstance(y_result, pdml.ModelSeries)
        tm.assert_index_equal(y_result.index, tdf.index)

        self.assertIsInstance(std_result, pdml.ModelSeries)
        tm.assert_index_equal(std_result.index, tdf.index)

        self.assert_numpy_array_almost_equal(y_result.values, y_expected)
        self.assert_numpy_array_almost_equal(std_result.values,
                                             std_expected)

    def test_Gaussian2D(self):
        # http://scikit-learn.org/stable/auto_examples/gaussian_process/plot_gp_probabilistic_classification_after_regression.html

        def g(x):
            """The function to predict (classification will then consist in predicting
            whether g(x) <= 0 or not)"""
            return 5. - x[:, 1] - .5 * x[:, 0] ** 2.

        # Design of experiments
        X = np.array([[-4.61611719, -6.00099547],
                      [4.10469096, 5.32782448],
                      [0.00000000, -0.50000000],
                      [-6.17289014, -4.6984743],
                      [1.3109306, -6.93271427],
                      [-5.03823144, 3.10584743],
                      [-2.87600388, 6.74310541],
                      [5.21301203, 4.26386883]])
        y = g(X)

        df = pdml.ModelFrame(X, target=y)
        gpm1 = df.gaussian_process.GaussianProcessRegressor()
        df.fit(gpm1)
        result = df.predict(gpm1)

        gpm2 = gp.GaussianProcessRegressor()
        gpm2.fit(X, y)
        expected = gpm2.predict(X)

        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_Gaussian2D_std(self):
        # http://scikit-learn.org/stable/auto_examples/gaussian_process/plot_gp_probabilistic_classification_after_regression.html

        def g(x):
            """The function to predict (classification will then consist in predicting
            whether g(x) <= 0 or not)"""
            return 5. - x[:, 1] - .5 * x[:, 0] ** 2.

        # Design of experiments
        X = np.array([[-4.61611719, -6.00099547],
                      [4.10469096, 5.32782448],
                      [0.00000000, -0.50000000],
                      [-6.17289014, -4.6984743],
                      [1.3109306, -6.93271427],
                      [-5.03823144, 3.10584743],
                      [-2.87600388, 6.74310541],
                      [5.21301203, 4.26386883]])
        y = g(X)

        df = pdml.ModelFrame(X, target=y)
        gpm1 = df.gaussian_process.GaussianProcessRegressor()
        df.fit(gpm1)
        result, std_result = df.predict(gpm1, return_std=True)

        gpm2 = gp.GaussianProcessRegressor()
        gpm2.fit(X, y)
        expected, std_expected = gpm2.predict(X, return_std=True)

        self.assert_numpy_array_almost_equal(result.values, expected)
        self.assert_numpy_array_almost_equal(std_result.values, std_expected)



#!/usr/bin/env python

from pandas_ml.compat import cache_readonly
from pandas_ml.core.accessor import _AccessorMethods, _attach_methods, _wrap_data_func


class ClusterMethods(_AccessorMethods):
    """Accessor to ``sklearn.cluster``."""

    _module_name = 'sklearn.cluster'

    def k_means(self, n_clusters, *args, **kwargs):
        """
        Call ``sklearn.cluster.k_means`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.k_means
        data = self._data
        centroid, label, inertia = func(data.values, n_clusters, *args, **kwargs)
        label = self._constructor_sliced(label, index=data.index)
        return centroid, label, inertia

    def affinity_propagation(self, *args, **kwargs):
        """
        Call ``sklearn.cluster.affinity_propagation`` using automatic mapping.

        - ``S``: ``ModelFrame.data``
        """
        func = self._module.affinity_propagation
        data = self._data
        cluster_centers_indices, labels = func(data.values, *args, **kwargs)
        labels = self._constructor_sliced(labels, index=data.index)
        return cluster_centers_indices, labels

    def dbscan(self, *args, **kwargs):
        """
        Call ``sklearn.cluster.dbscan`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.dbscan
        data = self._data
        core_samples, labels = func(data.values, *args, **kwargs)
        labels = self._constructor_sliced(labels, index=data.index)
        return core_samples, labels

    def mean_shift(self, *args, **kwargs):
        """
        Call ``sklearn.cluster.mean_shift`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.mean_shift
        data = self._data
        cluster_centers, labels = func(data.values, *args, **kwargs)
        labels = self._constructor_sliced(labels, index=data.index)
        return cluster_centers, labels

    def spectral_clustering(self, *args, **kwargs):
        """
        Call ``sklearn.cluster.spectral_clustering`` using automatic mapping.

        - ``affinity``: ``ModelFrame.data``
        """
        func = self._module.spectral_clustering
        data = self._data
        labels = func(data.values, *args, **kwargs)
        labels = self._constructor_sliced(labels, index=data.index)
        return labels

    # Biclustering

    @property
    def bicluster(self):
        """Property to access ``sklearn.cluster.bicluster``"""
        return self._bicluster

    @cache_readonly
    def _bicluster(self):
        return _AccessorMethods(self._df, module_name='sklearn.cluster.bicluster')


_cluster_methods = ['estimate_bandwidth', 'ward_tree']
_attach_methods(ClusterMethods, _wrap_data_func, _cluster_methods)



#!/usr/bin/env python

import sklearn.calibration as calibration

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestCalibration(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.calibration.CalibratedClassifierCV,
                      calibration.CalibratedClassifierCV)



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class EnsembleMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.ensemble``.
    """

    _module_name = 'sklearn.ensemble'

    @property
    def partial_dependence(self):
        """Property to access ``sklearn.ensemble.partial_dependence``"""
        # unable to set with cache_readonly
        return PartialDependenceMethods(self._df)


class PartialDependenceMethods(_AccessorMethods):
    # _module_name = 'sklearn.ensemble.partial_dependence'
    # 'sklearn.ensemble.partial_dependence' has no attribute '__all__'

    def partial_dependence(self, gbrt, target_variables, **kwargs):
        """
        Call ``sklearn.ensemble.partial_dependence`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        import sklearn.ensemble.partial_dependence as pdp
        func = pdp.partial_dependence
        pdp, axes = func(gbrt, target_variables, X=self._data, **kwargs)
        return pdp, axes

    def plot_partial_dependence(self, gbrt, features, **kwargs):
        """
        Call ``sklearn.ensemble.plot_partial_dependence`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        import sklearn.ensemble.partial_dependence as pdp
        func = pdp.plot_partial_dependence
        data = self._data
        fig, axes = func(gbrt, X=data, features=features, **kwargs)
        return fig, axes



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
try:
    import sklearn.discriminant_analysis as da
except ImportError:
    pass

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestDiscriminantAnalysis(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.discriminant_analysis.LinearDiscriminantAnalysis,
                      da.LinearDiscriminantAnalysis)
        self.assertIs(df.discriminant_analysis.QuadraticDiscriminantAnalysis,
                      da.QuadraticDiscriminantAnalysis)

        self.assertIs(df.da.LinearDiscriminantAnalysis,
                      da.LinearDiscriminantAnalysis)
        self.assertIs(df.da.QuadraticDiscriminantAnalysis,
                      da.QuadraticDiscriminantAnalysis)

    def test_objectmapper_deprecated(self):
        df = pdml.ModelFrame([])
        with tm.assert_produces_warning(FutureWarning):
            self.assertIs(df.lda.LinearDiscriminantAnalysis,
                          da.LinearDiscriminantAnalysis)
        with tm.assert_produces_warning(FutureWarning):
            self.assertIs(df.qda.QuadraticDiscriminantAnalysis,
                          da.QuadraticDiscriminantAnalysis)

    @pytest.mark.parametrize("algo", ['LinearDiscriminantAnalysis'])
    def test_LDA(self, algo):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        mod1 = getattr(df.da, algo)()
        mod2 = getattr(da, algo)()

        df.fit(mod1)
        mod2.fit(diabetes.data, diabetes.target)

        result = df.predict(mod1)
        expected = mod2.predict(diabetes.data)
        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python
import pytest

import numpy as np
import pandas as pd
import sklearn.cross_decomposition as cd

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestCrossDecomposition(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.cross_decomposition.PLSRegression, cd.PLSRegression)
        self.assertIs(df.cross_decomposition.PLSCanonical, cd.PLSCanonical)
        self.assertIs(df.cross_decomposition.CCA, cd.CCA)
        self.assertIs(df.cross_decomposition.PLSSVD, cd.PLSSVD)

    def test_CCA(self):
        X = [[0., 0., 1.], [1., 0., 0.], [2., 2., 2.], [3., 5., 4.]]
        Y = [[0.1, -0.2], [0.9, 1.1], [6.2, 5.9], [11.9, 12.3]]
        df = pdml.ModelFrame(X, target=Y)

        mod1 = df.cross_decomposition.CCA(n_components=1)
        mod2 = cd.CCA(n_components=1)

        df.fit(mod1)
        mod2.fit(X, Y)

        # 2nd cols are different on travis-CI
        self.assert_numpy_array_almost_equal(mod1.x_weights_[:, 0],
                                             mod2.x_weights_[:, 0])
        self.assert_numpy_array_almost_equal(mod1.y_weights_[:, 0],
                                             mod2.y_weights_[:, 0])

        result = df.transform(mod1)
        expected = mod2.transform(X, Y)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values.reshape(4),
                                             expected[0].reshape(4))
        self.assert_numpy_array_almost_equal(result.target.values.reshape(4),
                                             expected[1].reshape(4))

    @pytest.mark.parametrize("algo", ['CCA', 'PLSCanonical'])
    def test_CCA_PLSCannonical(self, algo):
        n = 500

        with tm.RNGContext(1):
            # 2 latents vars:
            l1 = np.random.normal(size=n)
            l2 = np.random.normal(size=n)

            latents = np.array([l1, l1, l2, l2]).T
            X = latents + np.random.normal(size=4 * n).reshape((n, 4))
            Y = latents + np.random.normal(size=4 * n).reshape((n, 4))

        X_train = X[:n // 2]
        Y_train = Y[:n // 2]
        X_test = X[n // 2:]
        Y_test = Y[n // 2:]

        train = pdml.ModelFrame(X_train, target=Y_train)
        test = pdml.ModelFrame(X_test, target=Y_test)

        # check multi target columns
        self.assertTrue(train.has_target())
        tm.assert_numpy_array_equal(train.data.values, X_train)
        tm.assert_numpy_array_equal(train.target.values, Y_train)
        tm.assert_numpy_array_equal(test.data.values, X_test)
        tm.assert_numpy_array_equal(test.target.values, Y_test)
        expected = pd.MultiIndex.from_tuples([('.target', 0), ('.target', 1),
                                              ('.target', 2), ('.target', 3)])
        tm.assert_index_equal(train.target_name, expected)
        self.assertEqual(train.data.shape, X_train.shape)
        self.assertEqual(train.target.shape, Y_train.shape)

        mod1 = getattr(train.cross_decomposition, algo)(n_components=2)
        mod2 = getattr(cd, algo)(n_components=2)

        train.fit(mod1)
        mod2.fit(X_train, Y_train)

        # 2nd cols are different on travis-CI
        self.assert_numpy_array_almost_equal(mod1.x_weights_[:, 0],
                                             mod2.x_weights_[:, 0])
        self.assert_numpy_array_almost_equal(mod1.y_weights_[:, 0],
                                             mod2.y_weights_[:, 0])

        result_tr = train.transform(mod1)
        result_test = test.transform(mod1)

        expected_tr = mod2.transform(X_train, Y_train)
        expected_test = mod2.transform(X_test, Y_test)

        self.assertIsInstance(result_tr, pdml.ModelFrame)
        self.assertIsInstance(result_test, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result_tr.data.values[:, 0],
                                             expected_tr[0][:, 0])
        self.assert_numpy_array_almost_equal(result_tr.target.values[:, 0],
                                             expected_tr[1][:, 0])
        self.assert_numpy_array_almost_equal(result_test.data.values[:, 0],
                                             expected_test[0][:, 0])
        self.assert_numpy_array_almost_equal(result_test.target.values[:, 0],
                                             expected_test[1][:, 0])

    def test_PLSRegression(self):

        n = 1000
        q = 3
        p = 10
        X = np.random.normal(size=n * p).reshape((n, p))
        B = np.array([[1, 2] + [0] * (p - 2)] * q).T
        # each Yj = 1*X1 + 2*X2 + noize
        Y = np.dot(X, B) + np.random.normal(size=n * q).reshape((n, q)) + 5

        df = pdml.ModelFrame(X, target=Y)
        pls1 = df.cross_decomposition.PLSRegression(n_components=3)
        df.fit(pls1)
        result = df.predict(pls1)

        pls2 = cd.PLSRegression(n_components=3)
        pls2.fit(X, Y)
        expected = pls2.predict(X)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/python
# -*- coding: utf8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import math
import numpy as np
import pandas as pd
import collections

from pandas_ml.confusion_matrix.abstract import ConfusionMatrixAbstract


class BinaryConfusionMatrix(ConfusionMatrixAbstract):
    """
    Binary confusion matrix class
    """
    def __init__(self, *args, **kwargs):
        # super(BinaryConfusionMatrix, self).__init__(y_true, y_pred)
        super(BinaryConfusionMatrix, self).__init__(*args, **kwargs)
        assert self.len() == 2, \
            "Binary confusion matrix must have len=2 but \
len=%d because y_true.unique()=%s y_pred.unique()=%s" \
% (self.len(), self.y_true().unique(), self.y_pred().unique())

    @classmethod
    def help(cls):
        """
        Returns a DataFrame reminder about terms
        * TN: True Negative
        * FP: False Positive
        * FN: False Negative
        * TP: True Positive
        """
        df = pd.DataFrame([["TN", "FP"], ["FN", "TP"]],
                          columns=[False, True], index=[False, True])
        df.index.name = cls.TRUE_NAME
        df.columns.name = cls.PRED_NAME
        return(df)

    @property
    def is_binary(self):
        """Return True"""
        return(True)

    def _class(self, direction):
        """Returns class for a given direction
        direction being a boolean
        True for positive class
        False for negative class"""
        if direction:
            return(self.pos_class)
        else:
            return(self.neg_class)

    @property
    def pos_class(self):
        """Returns positive class
        If BinaryConfusionMatrix was instantiate using y_true and y_pred
        as array of booleans, it should return True
        Else it should return the name (string) of the positive class"""
        return(self.classes[1])

    @property
    def neg_class(self):
        """Returns negative class
        If BinaryConfusionMatrix was instantiate using y_true and y_pred
        as array of booleans, it should return False
        Else it should return the name (string) of the negative class"""
        return(self.classes[0])

    def dict_class(self, reversed=False):
        if not reversed:
            d = {
                self.classes[0]: False,
                self.classes[1]: True,
            }
        else:
            d = {
                False: self.classes[0],
                True: self.classes[1]
            }
        return(d)

    def y_true(self, to_bool=False):
        if not to_bool:
            return(self._y_true)
        else:
            d = self.dict_class()
            return(self._y_true.map(d))

    def y_pred(self, to_bool=False):
        if not to_bool:
            return(self._y_pred)
        else:
            d = self.dict_class()
            return(self._y_pred.map(d))

    @property
    def P(self):
        """Condition positive
        eqv. with support"""
        return(self._df_confusion.loc[self._class(True), :].sum())

    @property
    def support(self):
        """
        same as P
        """
        return(self.P)

    @property
    def N(self):
        """Condition negative"""
        return(self._df_confusion.loc[self._class(False), :].sum())

    @property
    def TP(self):
        """
        true positive (TP)
        eqv. with hit
        """
        return(self._df_confusion.loc[self._class(True), self._class(True)])

    @property
    def hit(self):
        """
        same as TP
        """
        return(self.TP)

    @property
    def TN(self):
        """
        true negative (TN)
        eqv. with correct rejection
        """
        return(self._df_confusion.loc[self._class(False), self._class(False)])

    @property
    def FN(self):
        """
        false negative (FN)
        eqv. with miss, Type II error / Type 2 error
        """
        return(self._df_confusion.loc[self._class(True), self._class(False)])

    @property
    def FP(self):
        """
        false positive (FP)
        eqv. with false alarm, Type I error / Type 1 error
        """
        return(self._df_confusion.loc[self._class(False), self._class(True)])

    @property
    def PositiveTest(self):
        """
        test outcome positive
        TP + FP
        """
        return(self.TP + self.FP)

    @property
    def NegativeTest(self):
        """
        test outcome negative
        TN + FN
        """
        return(self.TN + self.FN)

    @property
    def FPR(self):
        """
        false positive rate (FPR)
        eqv. with fall-out
        FPR = FP / N = FP / (FP + TN)
        """
        # return(np.float64(self.FP)/(self.FP + self.TN))
        return(np.float64(self.FP) / self.N)

    @property
    def TPR(self):
        """
        true positive rate (TPR)
        eqv. with hit rate, recall, sensitivity
        TPR = TP / P = TP / (TP+FN)
        """
        # return(np.float64(self.TP) / (self.TP + self.FN))
        return(np.float64(self.TP) / self.P)

    @property
    def recall(self):
        """
        same as TPR
        """
        return(self.TPR)

    @property
    def sensitivity(self):
        """
        same as TPR
        """
        return(self.TPR)

    @property
    def TNR(self):
        """
        specificity (SPC) or true negative rate (TNR)
        SPC = TN / N = TN / (FP + TN)
        """
        return(np.float64(self.TN) / self.N)

    @property
    def SPC(self):
        """
        same as TNR
        """
        return(self.TNR)

    @property
    def specificity(self):
        """
        same as TNR
        """
        return(self.TNR)

    @property
    def PPV(self):
        """
        positive predictive value (PPV)
        eqv. with precision
        PPV = TP / (TP + FP) = TP / PositiveTest
        """
        return(np.float64(self.TP) / self.PositiveTest)

    @property
    def precision(self):
        """
        same as PPV
        """
        return(self.PPV)

    @property
    def FOR(self):
        """
        false omission rate (FOR)
        FOR = FN / NegativeTest
        """
        return(np.float64(self.FN) / self.NegativeTest)

    @property
    def NPV(self):
        """
        negative predictive value (NPV)
        NPV = TN / (TN + FN)
        """
        return(np.float64(self.TN) / self.NegativeTest)

    @property
    def FDR(self):
        """
        false discovery rate (FDR)
        FDR = FP / (FP + TP) = 1 - PPV
        """
        return(np.float64(self.FP) / self.PositiveTest)
        # return(1 - self.PPV)

    @property
    def FNR(self):
        """
        Miss Rate or False Negative Rate (FNR)
        FNR = FN / P = FN / (FN + TP)
        """
        return(np.float64(self.FN) / self.P)

    @property
    def ACC(self):
        """
        accuracy (ACC)
        ACC = (TP + TN) / (P + N) = (TP + TN) / TotalPopulation
        """
        return(np.float64(self.TP + self.TN) / self.population)

    @property
    def F1_score(self):
        """
        F1 score is the harmonic mean of precision and sensitivity
        F1 = 2 TP / (2 TP + FP + FN)
        can be also F1 = 2 * (precision * recall) / (precision + recall)
        """
        return(2 * np.float64(self.TP) / (2 * self.TP + self.FP + self.FN))

    @property
    def MCC(self):
        """
        Matthews correlation coefficient (MCC)
        """
        return((self.TP * self.TN - self.FP * self.FN)
               / math.sqrt((self.TP + self.FP) * (self.TP + self.FN)
               * (self.TN + self.FP) * (self.TN + self.FN)))

    @property
    def informedness(self):
        """
        Informedness = Sensitivity + Specificity - 1
        """
        return(self.sensitivity + self.specificity - 1.0)

    @property
    def markedness(self):
        """
        Markedness = Precision + NPV - 1
        """
        return(self.precision + self.NPV - 1.0)

    @property
    def prevalence(self):
        """
        Prevalence = P / TotalPopulation
        """
        return(np.float64(self.P) / self.population)

    @property
    def LRP(self):
        """
        Positive likelihood ratio (LR+) = TPR / FPR
        """
        return(np.float64(self.TPR) / self.FPR)

    @property
    def LRN(self):
        """
        Negative likelihood ratio (LR-) = FNR / TNR
        """
        return(np.float64(self.FNR) / self.TNR)

    @property
    def DOR(self):
        """
        Diagnostic odds ratio (DOR) = LR+ / LR−
        """
        return(np.float64(self.LRP) / self.LRN)

    def stats(self, lst_stats=None):
        """
        Returns an ordered dictionary of statistics
        """
        if lst_stats is None:
            lst_stats = [
                'population', 'P', 'N', 'PositiveTest', 'NegativeTest',
                'TP', 'TN', 'FP', 'FN', 'TPR', 'TNR', 'PPV', 'NPV', 'FPR', 'FDR',
                'FNR', 'ACC', 'F1_score', 'MCC', 'informedness', 'markedness',
                'prevalence', 'LRP', 'LRN', 'DOR', 'FOR']
        d = map(lambda stat: (stat, getattr(self, stat)), lst_stats)
        return(collections.OrderedDict(d))

    def _str_stats(self, lst_stats=None):
        """
        Returns a string representation of statistics
        """
        return(self._str_dict(self.stats(lst_stats),
               line_feed_key_val=' ', line_feed_stats='\n', d_name=None))

    def inverse(self):
        """
        Inverses a binary confusion matrix
        False -> True
        True -> False
        """
        negative_class = self.classes[0]  # False
        return(self.binarize(negative_class))



#!/usr/bin/env python

import warnings

import pandas as pd

from pandas_ml.core.accessor import _AccessorMethods


class ModelSelectionMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.model_selection``.
    """

    _module_name = 'sklearn.model_selection'

    # Splitter Classes

    def StratifiedShuffleSplit(self, *args, **kwargs):
        """
        Instanciate ``sklearn.cross_validation.StratifiedShuffleSplit`` using automatic mapping.

        - ``y``: ``ModelFrame.target``
        """
        return self._module.StratifiedShuffleSplit(*args, **kwargs)

    def split(self, cv, reset_index=False):
        """
        Generate ``ModelFrame`` using iterators for cross validation

        Parameters
        ----------
        cv : cross validation iterator
        reset_index : bool
            logical value whether to reset index, default False

        Returns
        -------
        generated : generator of ``ModelFrame``
        """
        if not(isinstance(cv, self._module.BaseCrossValidator)):
            msg = "{0} is not a subclass of BaseCrossValidator"
            warnings.warn(msg.format(cv.__class__.__name__))

        if isinstance(cv, self._module.StratifiedShuffleSplit):
            gen = cv.split(self._df.data.values, self._df.target.values)
        else:
            gen = cv.split(self._df.index)

        for train_index, test_index in gen:
            train_df = self._df.iloc[train_index, :]
            test_df = self._df.iloc[test_index, :]
            if reset_index:
                train_df = train_df.reset_index(drop=True)
                test_df = test_df.reset_index(drop=True)
            yield train_df, test_df

    def iterate(self, cv, reset_index=False):
        """ deprecated. Use .split """
        warnings.warn(".iterate is deprecated. Use .split instead",
                      FutureWarning, stacklevel=2)
        return self.split(cv, reset_index=reset_index)

    # Splitter Functions

    def train_test_split(self, reset_index=False, *args, **kwargs):
        """
        Call ``sklearn.cross_validation.train_test_split`` using automatic mapping.

        Parameters
        ----------
        reset_index : bool
            logical value whether to reset index, default False
        kwargs : keywords passed to ``cross_validation.train_test_split``

        Returns
        -------
        train, test : tuple of ``ModelFrame``
        """
        func = self._module.train_test_split

        def _init(klass, data, index, **kwargs):
            if reset_index:
                return klass(data, **kwargs)
            else:
                return klass(data, index=index, **kwargs)

        data = self._data
        idx = self._df.index
        if self._df.has_target():
            target = self._target
            tr_d, te_d, tr_l, te_l, tr_i, te_i = func(data.values, target.values, idx.values,
                                                      *args, **kwargs)

            # Create DataFrame here to retain data and target names
            tr_d = _init(pd.DataFrame, tr_d, tr_i, columns=data.columns)
            te_d = _init(pd.DataFrame, te_d, te_i, columns=data.columns)
            tr_l = _init(pd.Series, tr_l, tr_i, name=target.name)
            te_l = _init(pd.Series, te_l, te_i, name=target.name)

            train_df = self._constructor(data=tr_d, target=tr_l)
            test_df = self._constructor(data=te_d, target=te_l)
            return train_df, test_df
        else:
            tr_d, te_d, tr_i, te_i = func(data.values, idx.values, *args, **kwargs)

            # Create DataFrame here to retain data and target names
            tr_d = _init(pd.DataFrame, tr_d, tr_i, columns=data.columns)
            te_d = _init(pd.DataFrame, te_d, te_i, columns=data.columns)

            train_df = self._constructor(data=tr_d)
            train_df.target_name = self._df.target_name
            test_df = self._constructor(data=te_d)
            test_df.target_name = self._df.target_name
            return train_df, test_df

    # Hyper-parameter optimizers

    def describe(self, estimator):
        """
        Describe grid search results

        Parameters
        ----------
        estimator : fitted grid search estimator

        Returns
        -------
        described : ``ModelFrame``
        """
        return self._constructor(estimator.cv_results_)

    # Model validation

    def cross_val_score(self, estimator, *args, **kwargs):
        """
        Call ``sklearn.cross_validation.cross_val_score`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.cross_val_score
        return func(estimator, X=self._data.values, y=self._target.values, *args, **kwargs)

    def permutation_test_score(self, estimator, *args, **kwargs):
        """
        Call ``sklearn.cross_validation.permutation_test_score`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.permutation_test_score
        score, pscores, pvalue = func(estimator, X=self._data.values, y=self._target.values,
                                      *args, **kwargs)
        return score, pscores, pvalue

    def check_cv(self, cv, *args, **kwargs):
        """
        Call ``sklearn.cross_validation.check_cv`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.check_cv
        return func(cv, y=self._target, *args, **kwargs)

    def learning_curve(self, estimator, *args, **kwargs):
        """
        Call ``sklearn.lerning_curve.learning_curve`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.learning_curve
        data = self._data
        target = self._target
        tr_size, tr_score, te_score = func(estimator, X=data.values, y=target.values,
                                           *args, **kwargs)
        return tr_size, tr_score, te_score

    def validation_curve(self, estimator, param_name, param_range, *args, **kwargs):
        """
        Call ``sklearn.learning_curve.validation_curve`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.validation_curve
        data = self._data
        target = self._target
        tr_score, te_score = func(estimator, X=data.values, y=target.values,
                                  param_name=param_name, param_range=param_range,
                                  *args, **kwargs)
        return tr_score, te_score



#!/usr/bin/env python
import pytest

import numpy as np
import sklearn.datasets as datasets
import sklearn.svm as svm

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestSVM(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.svm.SVC, svm.SVC)
        self.assertIs(df.svm.LinearSVC, svm.LinearSVC)
        self.assertIs(df.svm.NuSVC, svm.NuSVC)
        self.assertIs(df.svm.SVR, svm.SVR)
        self.assertIs(df.svm.NuSVR, svm.NuSVR)
        self.assertIs(df.svm.OneClassSVM, svm.OneClassSVM)

    def test_l1_min_c(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.svm.l1_min_c()
        expected = svm.l1_min_c(iris.data, iris.target)
        self.assertAlmostEqual(result, expected)

    @pytest.mark.parametrize("algo", ['SVR', 'NuSVR'])
    def test_Regressions_curve(self, algo):
        # http://scikit-learn.org/stable/auto_examples/plot_kernel_ridge_regression.html
        X = 5 * np.random.rand(1000, 1)
        y = np.sin(X).ravel()

        # Add noise to targets
        y[::5] += 3 * (0.5 - np.random.rand(X.shape[0] // 5))

        df = pdml.ModelFrame(data=X, target=y)

        mod1 = getattr(df.svm, algo)()
        mod2 = getattr(svm, algo)()

        df.fit(mod1)
        mod2.fit(X, y)

        result = df.predict(mod1)
        expected = mod2.predict(X)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        self.assertIsInstance(df.predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(df.predicted.values, expected)

    @pytest.mark.parametrize("algo", ['SVR', 'NuSVR'])
    def test_Regressions_iris(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.svm, algo)()
        mod2 = getattr(svm, algo)()

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        self.assertIsInstance(df.predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(df.predicted.values, expected)

    @pytest.mark.parametrize("algo", ['LinearSVC', 'NuSVC'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.svm, algo)(random_state=self.random_state)
        mod2 = getattr(svm, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python

import warnings

import numpy as np
import pandas as pd
import pandas.compat as compat

import pandas_ml.imbaccessors as imbaccessors
import pandas_ml.skaccessors as skaccessors
import pandas_ml.smaccessors as smaccessors
import pandas_ml.snsaccessors as snsaccessors
import pandas_ml.util as util
import pandas_ml.xgboost as xgboost
from pandas_ml.compat import is_list_like, Appender, cache_readonly
from pandas_ml.core.accessor import _AccessorMethods
from pandas_ml.core.generic import ModelPredictor, _shared_docs
from pandas_ml.core.series import ModelSeries


class ModelFrame(ModelPredictor, pd.DataFrame):
    """
    Data structure subclassing ``pandas.DataFrame`` to define a metadata to
    specify target (response variable) and data (explanatory variable / features).

    Parameters
    ----------
    data : same as ``pandas.DataFrame``
    target : str or array-like
        Column name or values to be used as target
    args : arguments passed to ``pandas.DataFrame``
    kwargs : keyword arguments passed to ``pandas.DataFrame``
    """

    _internal_caches = ['_estimator', '_predicted', '_proba', '_log_proba', '_decision']
    _internal_names = (pd.core.generic.NDFrame._internal_names + _internal_caches)
    _internal_names_set = set(_internal_names)
    _metadata = ['_target_name']

    _method_mapper = dict(fit={}, transform={}, predict={})
    for cls in [skaccessors.CrossDecompositionMethods,
                skaccessors.GaussianProcessMethods]:
        _method_mapper = cls._update_method_mapper(_method_mapper)

    @property
    def _constructor(self):
        return ModelFrame

    _constructor_sliced = ModelSeries

    _TARGET_NAME = '.target'
    _DATA_NAME = '.data'

    def __init__(self, data, target=None,
                 *args, **kwargs):

        if data is None and target is None:
            msg = '{0} must have either data or target'
            raise ValueError(msg.format(self.__class__.__name__))
        elif data is None and not is_list_like(target):
            msg = 'target must be list-like when data is None'
            raise ValueError(msg)

        data, target = skaccessors._maybe_sklearn_data(data, target)
        data, target = smaccessors._maybe_statsmodels_data(data, target)

        # retrieve target_name
        if isinstance(data, ModelFrame):
            target_name = data.target_name

        data, target = self._maybe_convert_data(data, target, *args, **kwargs)

        if target is not None and not is_list_like(target):
            if target in data.columns:
                target_name = target
                df = data
            else:
                msg = "Specified target '{0}' is not included in data"
                raise ValueError(msg.format(target))
            self._target_name = target_name
        else:
            df, target = self._concat_target(data, target)

            if isinstance(target, pd.Series):
                self._target_name = target.name

            elif isinstance(target, pd.DataFrame):
                if len(target.columns) > 1:
                    self._target_name = target.columns
                else:
                    self._target_name = target.columns[0]
            else:
                # target may be None
                self._target_name = self._TARGET_NAME

        pd.DataFrame.__init__(self, df)

    def _maybe_convert_data(self, data, target,
                            *args, **kwargs):
        """
        Internal function to instanciate data and target

        Parameters
        ----------
        data : instance converted to ``pandas.DataFrame``
        target : instance converted to ``pandas.Series``
        args : argument passed from ``__init__``
        kwargs : argument passed from ``__init__``
        """

        init_df = isinstance(data, pd.DataFrame)
        init_target = isinstance(target, (pd.Series, pd.DataFrame))

        def _maybe_convert_target(data, target, index=None):
            if data is not None:
                index = data.index

            target = np.array(target)
            if len(target.shape) == 1:
                target = pd.Series(target, index=index)
            else:
                target = pd.DataFrame(target, index=index)
            return target

        if not init_df and not init_target:
            if data is not None:
                data = pd.DataFrame(data, *args, **kwargs)

            if is_list_like(target):
                target = _maybe_convert_target(data, target)

        elif not init_df:
            if data is not None:
                index = kwargs.pop('index', target.index)
                data = pd.DataFrame(data, index=index, *args, **kwargs)

        elif not init_target:
            if is_list_like(target):
                target = _maybe_convert_target(data, target)

        else:
            # no conversion required
            pass

        if isinstance(target, pd.Series) and target.name is None:
            target = pd.Series(target, name=self._TARGET_NAME)

        return data, target

    def _concat_target(self, data, target):
        if data is None and target is None:
            msg = '{0} must have either data or target'
            raise ValueError(msg.format(self.__class__.__name__))

        elif data is None:
            return target, target

        elif target is None:
            return data, None

        if len(data) != len(target):
            raise ValueError('data and target must have same length')

        if not data.index.equals(target.index):
            raise ValueError('data and target must have equal index')

        def _add_meta_columns(df, meta_name):
            df = df.copy()
            if not is_list_like(meta_name):
                meta_name = [meta_name]
            df.columns = pd.MultiIndex.from_product([meta_name, df.columns])
            return df

        if isinstance(target, pd.DataFrame):
            if len(target.columns.intersection(data.columns)) > 0:
                target = _add_meta_columns(target, self._TARGET_NAME)
                data = _add_meta_columns(data, self._DATA_NAME)
                # overwrite target_name
                self._target_name = target.columns
        elif isinstance(target, pd.Series):
            if target.name in data.columns:
                raise ValueError('data and target must have unique names')
        else:
            raise ValueError('target cannot be converted to ModelSeries or ModelFrame')

        return pd.concat([target, data], axis=1), target

    def has_data(self):
        """
        Return whether ``ModelFrame`` has data

        Returns
        -------
        has_data : bool
        """
        return len(self._data_columns) > 0

    @property
    def _data_columns(self):
        # Index.difference results in sorted difference set
        if self.has_multi_targets():
            return self.columns[~self.columns.isin(self.target_name)]
        else:
            # This doesn't work for DatetimeIndex
            # return self.columns[~(self.columns == self.target_name)]
            return pd.Index([c for c in self.columns if c != self.target_name])

    @property
    def data(self):
        """
        Return data (explanatory variable / features)

        Returns
        -------
        data : ``ModelFrame``
        """
        if self.has_data():
            return self.loc[:, self._data_columns]
        else:
            return None

    @data.setter
    def data(self, data):
        if data is None:
            del self.data
            return

        if isinstance(data, ModelFrame):
            if data.has_target():
                msg = 'Cannot update with {0} which has target attribute'
                raise ValueError(msg.format(self.__class__.__name__))
        elif isinstance(data, pd.Series):
            data = data.to_frame()
        elif isinstance(data, pd.DataFrame):
            pass
        else:
            msg = 'data must be ModelFrame, ModelSeries, DataFrame or Series, {0} passed'
            raise TypeError(msg.format(data.__class__.__name__))

        data, _ = self._maybe_convert_data(data, self.target, self.target_name)

        if self.has_multi_targets():
            if len(self.target_name.intersection(data.columns)) > 0:
                msg = "Passed data has the same column name as the target '{0}'"
                raise ValueError(msg.format(self.target_name))
        else:
            if self.target_name in data.columns:
                msg = "Passed data has the same column name as the target '{0}'"
                raise ValueError(msg.format(self.target_name))

        if self.has_target():
            data, _ = self._concat_target(data, self.target)
        self._update_inplace(data)

    @data.deleter
    def data(self):
        if self.has_target():
            self._update_inplace(self.target.to_frame())
        else:
            msg = '{0} must have either data or target'
            raise ValueError(msg.format(self.__class__.__name__))

    def has_target(self):
        """
        Return whether ``ModelFrame`` has target

        Returns
        -------
        has_target : bool
        """
        if self.has_multi_targets():
            return len(self.target_name.intersection(self.columns)) > 0
        return self.target_name in self.columns

    def has_multi_targets(self):
        """
        Return whether ``ModelFrame`` has multiple target columns

        Returns
        -------
        has_multi_targets : bool
        """
        return isinstance(self.target_name, pd.Index)

    @property
    def target_name(self):
        """
        Return target column name

        Returns
        -------
        target : object
        """
        return self._target_name

    @target_name.setter
    def target_name(self, value):
        self._target_name = value

    @property
    def target(self):
        """
        Return target (response variable)

        Returns
        -------
        target : ``ModelSeries``
        """
        if self.has_target():
            return self.loc[:, self.target_name]
        else:
            return None

    @target.setter
    def target(self, target):
        if target is None:
            del self.target
            return

        if not self.has_target():
            # allow to update target_name only when target attibute doesn't exist
            if isinstance(target, pd.Series):
                # Series.name may be blank
                if target.name is not None:
                    self.target_name = target.name
            elif isinstance(target, pd.DataFrame):
                # DataFrame.columns should have values
                self.target_name = target.columns

        if not is_list_like(target):
            if target in self.columns:
                self.target_name = target
            else:
                msg = "Specified target '{0}' is not included in data"
                raise ValueError(msg.format(target))
            return

        if isinstance(target, pd.Series):
            if target.name != self.target_name:
                msg = "Passed data is being renamed to '{0}'".format(self.target_name)
                warnings.warn(msg)
                target = pd.Series(target, name=self.target_name)
        elif isinstance(target, pd.DataFrame):
            if not target.columns.equals(self.target_name):
                if len(target.columns) == len(self.target_name):
                    msg = "Passed data is being renamed to '{0}'".format(self.target_name)
                    warnings.warn(msg)
                    target = target.copy()
                    target.columns = self.target_name
                else:
                    msg = 'target and target_name are unmatched, target_name will be updated'
                    warnings.warn(msg)
                    data = self.data        # hack
                    self.target_name = target.columns
                    self.data = data
        else:
            _, target = self._maybe_convert_data(self.data, target, self.target_name)

        df, _ = self._concat_target(self.data, target)
        self._update_inplace(df)

    @target.deleter
    def target(self):
        if self.has_data():
            self._update_inplace(self.data)
        else:
            msg = '{0} must have either data or target'
            raise ValueError(msg.format(self.__class__.__name__))

    def _get_method_mapper(self, estimator, method_name):
        if method_name in self._method_mapper:
            mapper = self._method_mapper[method_name]
        return mapper.get(estimator.__class__.__name__, None)

    def _call(self, estimator, method_name, *args, **kwargs):
        method = self._check_attr(estimator, method_name)

        data = self.data.values
        if self.has_target():
            target = self.target.values
            try:
                result = method(data, y=target, *args, **kwargs)
            except TypeError:
                result = method(data, *args, **kwargs)
        else:
            # not try to pass target if it doesn't exists
            # to catch ValueError from estimator
            result = method(data, *args, **kwargs)
        self.estimator = estimator
        return result

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit_predict', returned='returned : predicted result'))
    def fit_predict(self, estimator, *args, **kwargs):
        predicted = self._call(estimator, 'fit_predict', *args, **kwargs)
        return self._wrap_predicted(predicted, estimator)

    def _wrap_predicted(self, predicted, estimator):
        """
        Wrapper for predict methods
        """
        if isinstance(predicted, tuple):
            return tuple(self._wrap_predicted(p, estimator) for p in predicted)
        if util._is_1d_varray(predicted):
            predicted = self._constructor_sliced(predicted, index=self.index)
        else:
            predicted = self._constructor(predicted, index=self.index)
        self._predicted = predicted
        return self._predicted

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit_sample', returned='returned : sampling result'))
    def fit_sample(self, estimator, *args, **kwargs):
        # for imblearn
        msg = ".fit_sample is deprecated. Use .fit_resample instead"
        warnings.warn(msg, DeprecationWarning)
        return self.fit_resample(estimator, *args, **kwargs)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit_resample', returned='returned : resampling result'))
    def fit_resample(self, estimator, *args, **kwargs):
        # for imblearn
        sampled_X, sampled_y = self._call(estimator, 'fit_resample', *args, **kwargs)
        return self._wrap_sampled(sampled_X, sampled_y)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='sample', returned='returned : sampling result'))
    def sample(self, estimator, *args, **kwargs):
        # for imblearn
        msg = ".sample is deprecated. Use .fit_resample instead"
        warnings.warn(msg, DeprecationWarning)
        return self.fit_resample(estimator, *args, **kwargs)

    def _wrap_sampled(self, sampled_X, sampled_y):
        # revert sampled results to ModelFrame, index is being reset

        def _wrap(x, y):
            y = self._constructor_sliced(y, name=self.target.name)
            result = self._constructor(data=x, target=y,
                                       columns=self.data.columns)
            return result

        if sampled_X.ndim == 3 or sampled_X.ndim == 1:
            # ensemble
            # ndim=3 for EasyEnsemble
            # ndim=1 for BalanceCascade
            results = []
            for x, y in zip(sampled_X, sampled_y):
                result = _wrap(x, y)
                results.append(result)
        else:
            results = _wrap(sampled_X, sampled_y)
        return results

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='transform', returned='returned : transformed result'))
    def transform(self, estimator, *args, **kwargs):
        try:
            transformed = super(ModelFrame, self).transform(estimator, *args, **kwargs)

            if not isinstance(estimator, compat.string_types):
                # set inverse columns
                estimator._pdml_original_columns = self.data.columns
            return transformed
        except ImportError:
            # raise patsy error
            raise
        except Exception as e:     # noqa
            return pd.DataFrame.transform(self, estimator, *args, **kwargs)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit_transform', returned='returned : transformed result'))
    def fit_transform(self, estimator, *args, **kwargs):
        transformed = super(ModelFrame, self).fit_transform(estimator, *args, **kwargs)

        if not isinstance(estimator, compat.string_types):
            # set inverse columns
            estimator._pdml_original_columns = self.data.columns
        return transformed

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='inverse_transform', returned='returned : transformed result'))
    def inverse_transform(self, estimator, *args, **kwargs):
        transformed = self._call(estimator, 'inverse_transform', *args, **kwargs)
        original_columns = getattr(estimator, '_pdml_original_columns', None)
        transformed = self._wrap_transform(transformed, columns=original_columns)
        return transformed

    def _wrap_transform(self, transformed, columns=None):
        """
        Wrapper for transform methods
        """
        if self.pp._keep_existing_columns(self.estimator):
            columns = self.data.columns

        if self.has_target():
            return self._constructor(transformed, target=self.target,
                                     index=self.index, columns=columns)
        else:
            return self._constructor(transformed, index=self.index,
                                     columns=columns)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='predict_proba', returned='returned : probabilities'))
    def predict_proba(self, estimator, *args, **kwargs):
        probability = self._call(estimator, 'predict_proba', *args, **kwargs)
        self._proba = self._wrap_probability(probability, estimator)
        return self._proba

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='predict_log_proba', returned='returned : probabilities'))
    def predict_log_proba(self, estimator, *args, **kwargs):
        probability = self._call(estimator, 'predict_log_proba', *args, **kwargs)
        self._log_proba = self._wrap_probability(probability, estimator)
        return self._log_proba

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='decision_function', returned='returned : decisions'))
    def decision_function(self, estimator, *args, **kwargs):
        decision = self._call(estimator, 'decision_function', *args, **kwargs)
        self._decision = self._wrap_probability(decision, estimator)
        return self._decision

    def _wrap_probability(self, probability, estimator):
        """
        Wrapper for probability methods
        """
        try:
            if util._is_1d_varray(probability):
                # 2 class
                probability = self._constructor(probability, index=self.index)
            else:
                probability = self._constructor(probability, index=self.index,
                                                columns=estimator.classes_)
        except ValueError:
            msg = "Unable to instantiate ModelFrame for '{0}'"
            warnings.warn(msg.format(estimator.__class__.__name__))
        return probability

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='score', returned='returned : score'))
    def score(self, estimator, *args, **kwargs):
        score = self._call(estimator, 'score', *args, **kwargs)
        return score

    # accessors

    @property
    @Appender(_shared_docs['skaccessor_nolink'] %
              dict(module='calibration'))
    def calibration(self):
        return self._calibration

    @cache_readonly
    def _calibration(self):
        attrs = ['CalibratedClassifierCV']
        return _AccessorMethods(self, module_name='sklearn.calibration',
                                attrs=attrs)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='cluster'))
    def cluster(self):
        return self._cluster

    @cache_readonly
    def _cluster(self):
        return skaccessors.ClusterMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='covariance'))
    def covariance(self):
        return self._covariance

    @cache_readonly
    def _covariance(self):
        return skaccessors.CovarianceMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='cross_decomposition'))
    def cross_decomposition(self):
        return self._cross_decomposition

    @cache_readonly
    def _cross_decomposition(self):
        attrs = ['PLSRegression', 'PLSCanonical', 'CCA', 'PLSSVD']
        return _AccessorMethods(self, module_name='sklearn.cross_decomposition',
                                attrs=attrs)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='decomposition'))
    def decomposition(self):
        return self._decomposition

    @cache_readonly
    def _decomposition(self):
        return skaccessors.DecompositionMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] %
              dict(module='discriminant_analysis'))
    def discriminant_analysis(self):
        return self._da

    @property
    @Appender(_shared_docs['skaccessor_nolink'] %
              dict(module='discriminant_analysis'))
    def da(self):
        return self._da

    @cache_readonly
    def _da(self):
        return _AccessorMethods(self,
                                module_name='sklearn.discriminant_analysis')

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='dummy'))
    def dummy(self):
        return self._dummy

    @cache_readonly
    def _dummy(self):
        attrs = ['DummyClassifier', 'DummyRegressor']
        return _AccessorMethods(self, module_name='sklearn.dummy', attrs=attrs)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='ensemble'))
    def ensemble(self):
        return self._ensemble

    @cache_readonly
    def _ensemble(self):
        return skaccessors.EnsembleMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='feature_extraction'))
    def feature_extraction(self):
        return self._feature_extraction

    @cache_readonly
    def _feature_extraction(self):
        return skaccessors.FeatureExtractionMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='feature_selection'))
    def feature_selection(self):
        return self._feature_selection

    @cache_readonly
    def _feature_selection(self):
        return skaccessors.FeatureSelectionMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='gaussian_process'))
    def gaussian_process(self):
        return self._gaussian_process

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='gaussian_process'))
    def gp(self):
        return self._gaussian_process

    @cache_readonly
    def _gaussian_process(self):
        return skaccessors.GaussianProcessMethods(self)

    @property
    def imbalance(self):
        """ Property to access ``imblearn``"""
        return self._imbalance

    @cache_readonly
    def _imbalance(self):
        return imbaccessors.ImbalanceMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='isotonic'))
    def isotonic(self):
        return self._isotonic

    @cache_readonly
    def _isotonic(self):
        return skaccessors.IsotonicMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='kernel_approximation'))
    def kernel_approximation(self):
        return self._kernel_approximation

    @cache_readonly
    def _kernel_approximation(self):
        attrs = ['AdditiveChi2Sampler', 'Nystroem', 'RBFSampler', 'SkewedChi2Sampler']
        return _AccessorMethods(self, module_name='sklearn.kernel_approximation',
                                attrs=attrs)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='kernel_ridge'))
    def kernel_ridge(self):
        return self._kernel_ridge

    @cache_readonly
    def _kernel_ridge(self):
        attrs = ['KernelRidge']
        return _AccessorMethods(self, module_name='sklearn.kernel_ridge',
                                attrs=attrs)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='lda'))
    def lda(self):
        msg = '.lda is deprecated. Use .da or .diccriminant_analysis'
        warnings.warn(msg, FutureWarning, stacklevel=2)
        return self._da

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='linear_model'))
    def linear_model(self):
        return self._linear_model

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='linear_model'))
    def lm(self):
        return self._linear_model

    @cache_readonly
    def _linear_model(self):
        return skaccessors.LinearModelMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='manifold'))
    def manifold(self):
        return self._manifold

    @cache_readonly
    def _manifold(self):
        return skaccessors.ManifoldMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='metrics'))
    def metrics(self):
        return self._metrics

    @cache_readonly
    def _metrics(self):
        return skaccessors.MetricsMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='mixture'))
    def mixture(self):
        return self._mixture

    @cache_readonly
    def _mixture(self):
        return _AccessorMethods(self, module_name='sklearn.mixture')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='model_selection'))
    def model_selection(self):
        return self._model_selection

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='model_selection'))
    def ms(self):
        return self._model_selection

    @cache_readonly
    def _model_selection(self):
        return skaccessors.ModelSelectionMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='multiclass'))
    def multiclass(self):
        return self._multiclass

    @cache_readonly
    def _multiclass(self):
        return _AccessorMethods(self, module_name='sklearn.multiclass')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='multioutput'))
    def multioutput(self):
        return self._multioutput

    @cache_readonly
    def _multioutput(self):
        return _AccessorMethods(self, module_name='sklearn.multioutput')

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='naive_bayes'))
    def naive_bayes(self):
        return self._naive_bayes

    @cache_readonly
    def _naive_bayes(self):
        return _AccessorMethods(self, module_name='sklearn.naive_bayes')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='neighbors'))
    def neighbors(self):
        return self._neighbors

    @cache_readonly
    def _neighbors(self):
        return skaccessors.NeighborsMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='neural_network'))
    def neural_network(self):
        return self._neural_network

    @cache_readonly
    def _neural_network(self):
        return _AccessorMethods(self, module_name='sklearn.neural_network')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='pipeline'))
    def pipeline(self):
        return self._pipeline

    @cache_readonly
    def _pipeline(self):
        return skaccessors.PipelineMethods(self)

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='preprocessing'))
    def preprocessing(self):
        return self._preprocessing

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='preprocessing'))
    def pp(self):
        return self.preprocessing

    @cache_readonly
    def _preprocessing(self):
        return skaccessors.PreprocessingMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='qda'))
    def qda(self):
        msg = '.qda is deprecated. Use .da or .diccriminant_analysis'
        warnings.warn(msg, FutureWarning, stacklevel=2)
        return self._da

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='random_projection'))
    def random_projection(self):
        return self._random_projection

    @cache_readonly
    def _random_projection(self):
        return _AccessorMethods(self, module_name='sklearn.random_projection')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='semi_supervised'))
    def semi_supervised(self):
        return self._semi_supervised

    @cache_readonly
    def _semi_supervised(self):
        return _AccessorMethods(self, module_name='sklearn.semi_supervised')

    @property
    @Appender(_shared_docs['skaccessor'] % dict(module='svm'))
    def svm(self):
        return self._svm

    @cache_readonly
    def _svm(self):
        return skaccessors.SVMMethods(self)

    @property
    @Appender(_shared_docs['skaccessor_nolink'] % dict(module='tree'))
    def tree(self):
        return self._tree

    @cache_readonly
    def _tree(self):
        return _AccessorMethods(self, module_name='sklearn.tree')

    @property
    def sns(self):
        """Property to access ``seaborn`` API"""
        return self._seaborn

    @property
    def seaborn(self):
        """Property to access ``seaborn`` API"""
        return self._seaborn

    @cache_readonly
    def _seaborn(self):
        return snsaccessors.SeabornMethods(self)

    @property
    def xgb(self):
        """Property to access ``xgboost.sklearn`` API"""
        return self._xgboost

    @property
    def xgboost(self):
        """Property to access ``xgboost.sklearn`` API"""
        return self._xgboost

    @cache_readonly
    def _xgboost(self):
        return xgboost.XGBoostMethods(self)

    @Appender(pd.core.generic.NDFrame.groupby.__doc__)
    def groupby(self, by=None, axis=0, level=None, as_index=True, sort=True,
                group_keys=True, squeeze=False):
        from pandas_ml.core.groupby import groupby
        if level is None and by is None:
            raise TypeError("You have to supply one of 'by' and 'level'")
        axis = self._get_axis_number(axis)
        return groupby(self, by=by, axis=axis, level=level, as_index=as_index,
                       sort=sort, group_keys=group_keys, squeeze=squeeze)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.neighbors as neighbors

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestNeighbors(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.neighbors.NearestNeighbors,
                      neighbors.NearestNeighbors)
        self.assertIs(df.neighbors.KNeighborsClassifier,
                      neighbors.KNeighborsClassifier)
        self.assertIs(df.neighbors.RadiusNeighborsClassifier,
                      neighbors.RadiusNeighborsClassifier)
        self.assertIs(df.neighbors.KNeighborsRegressor,
                      neighbors.KNeighborsRegressor)
        self.assertIs(df.neighbors.RadiusNeighborsRegressor,
                      neighbors.RadiusNeighborsRegressor)
        self.assertIs(df.neighbors.NearestCentroid, neighbors.NearestCentroid)
        self.assertIs(df.neighbors.BallTree, neighbors.BallTree)
        self.assertIs(df.neighbors.KDTree, neighbors.KDTree)
        self.assertIs(df.neighbors.DistanceMetric, neighbors.DistanceMetric)
        self.assertIs(df.neighbors.KernelDensity, neighbors.KernelDensity)

    def test_kneighbors_graph(self):
        x = [[0], [3], [1]]
        df = pdml.ModelFrame(x)

        result = df.neighbors.kneighbors_graph(2)
        expected = neighbors.kneighbors_graph(x, 2)

        self.assert_numpy_array_almost_equal(result.toarray(), expected.toarray())

    def test_radius_neighbors_graph(self):
        x = [[0], [3], [1]]
        df = pdml.ModelFrame(x)

        result = df.neighbors.radius_neighbors_graph(1.5)
        expected = neighbors.radius_neighbors_graph(x, 1.5)

        self.assert_numpy_array_almost_equal(result.toarray(), expected.toarray())

    @pytest.mark.parametrize("algo", ['NearestNeighbors',
                                      'KNeighborsRegressor'])
    def test_NearestNeigbors(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.neighbors, algo)(10)
        mod2 = getattr(neighbors, algo)(10)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        # df doesn't have kneighbors
        result = mod1.kneighbors(df.data)
        expected = mod2.kneighbors(iris.data)
        self.assert_numpy_array_almost_equal(result, expected)

    @pytest.mark.parametrize("algo", ['KNeighborsClassifier',
                                      'RadiusNeighborsRegressor',
                                      'NearestCentroid'])
    def test_Neigbors(self, algo):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        mod1 = getattr(df.neighbors, algo)()
        mod2 = getattr(neighbors, algo)()

        df.fit(mod1)
        mod2.fit(diabetes.data, diabetes.target)

        result = df.predict(mod1)
        expected = mod2.predict(diabetes.data)
        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.ensemble as ensemble
import sklearn.metrics as metrics

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestEnsemble(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.ensemble.AdaBoostClassifier,
                      ensemble.AdaBoostClassifier)
        self.assertIs(df.ensemble.AdaBoostRegressor,
                      ensemble.AdaBoostRegressor)
        self.assertIs(df.ensemble.BaggingClassifier,
                      ensemble.BaggingClassifier)
        self.assertIs(df.ensemble.BaggingRegressor,
                      ensemble.BaggingRegressor)
        self.assertIs(df.ensemble.ExtraTreesClassifier,
                      ensemble.ExtraTreesClassifier)
        self.assertIs(df.ensemble.ExtraTreesRegressor,
                      ensemble.ExtraTreesRegressor)

        self.assertIs(df.ensemble.GradientBoostingClassifier,
                      ensemble.GradientBoostingClassifier)
        self.assertIs(df.ensemble.GradientBoostingRegressor,
                      ensemble.GradientBoostingRegressor)

        self.assertIs(df.ensemble.IsolationForest,
                      ensemble.IsolationForest)

        self.assertIs(df.ensemble.RandomForestClassifier,
                      ensemble.RandomForestClassifier)
        self.assertIs(df.ensemble.RandomTreesEmbedding,
                      ensemble.RandomTreesEmbedding)
        self.assertIs(df.ensemble.RandomForestRegressor,
                      ensemble.RandomForestRegressor)

        self.assertIs(df.ensemble.VotingClassifier,
                      ensemble.VotingClassifier)

    @pytest.mark.parametrize("algo", ['AdaBoostClassifier',
                                      'BaggingClassifier',
                                      'RandomForestClassifier'])
    def test_Regressions(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.ensemble, algo)(random_state=self.random_state)
        mod2 = getattr(ensemble, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    @pytest.mark.parametrize("algo", ['AdaBoostClassifier',
                                      'BaggingClassifier',
                                      'RandomForestClassifier'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.ensemble, algo)(random_state=self.random_state)
        mod2 = getattr(ensemble, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_partial_dependence(self):
        samples = [[0, 0, 2], [1, 0, 0]]
        labels = [0, 1]

        df = pdml.ModelFrame(samples, target=labels)
        gb1 = df.ensemble.GradientBoostingClassifier(random_state=self.random_state)
        df.fit(gb1)
        result = df.ensemble.partial_dependence.partial_dependence(gb1, [0], percentiles=(0, 1),
                                                                   grid_resolution=2)

        gb2 = ensemble.GradientBoostingClassifier(random_state=self.random_state)
        gb2.fit(samples, labels)
        expected = ensemble.partial_dependence.partial_dependence(gb2, [0], X=samples, percentiles=(0, 1),
                                                                  grid_resolution=2)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])

    def test_plot_partial_dependence(self):
        df = pdml.ModelFrame(datasets.load_iris())
        clf = df.ensemble.GradientBoostingRegressor(n_estimators=10)
        df.fit(clf)
        """
        # ToDo: Check how to perform plotting test on travis, locally passed.
        fig, axes = df.ensemble.partial_dependence.plot_partial_dependence(clf, [0, (0, 1)])

        import matplotlib
        self.assertIsInstance(fig, matplotlib.figure.Figure)
        self.assertTrue(len(axes), 2)
        self.assertIsInstance(axes[0], matplotlib.axes.Axes)
        """

    def test_GradientBoostingRegression(self):
        boston = datasets.load_boston()
        df = pdml.ModelFrame(boston)

        params = {'n_estimators': 500, 'max_depth': 4, 'min_samples_split': 0.9,
                  'learning_rate': 0.01, 'loss': 'ls', 'random_state': self.random_state}
        clf1 = ensemble.GradientBoostingRegressor(**params)
        clf2 = df.ensemble.GradientBoostingRegressor(**params)

        clf1.fit(boston.data, boston.target)
        df.fit(clf2)

        expected = clf1.predict(boston.data)
        predicted = df.predict(clf2)
        self.assertIsInstance(predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(predicted.values, expected)

        self.assertAlmostEqual(df.metrics.mean_squared_error(),
                               metrics.mean_squared_error(boston.target, expected))



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class SVMMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.svm``.
    """

    _module_name = 'sklearn.svm'

    def l1_min_c(self, *args, **kwargs):
        """
        Call ``sklearn.svm.l1_min_c`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.l1_min_c
        data = self._data
        target = self._target
        l1_min_c = func(data.values, y=target.values, *args, **kwargs)
        return l1_min_c

    @property
    def libsvm(self):
        """Not implemented"""
        raise NotImplementedError

    @property
    def liblinear(self):
        """Not implemented"""
        raise NotImplementedError

    @property
    def libsvm_sparse(self):
        """Not implemented"""
        raise NotImplementedError



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class DecompositionMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.decomposition``.
    """

    _module_name = 'sklearn.decomposition'

    def fastica(self, *args, **kwargs):
        """
        Call ``sklearn.decomposition.fastica`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.fastica
        data = self._data
        return_x_mean = kwargs.get('return_X_mean', False)

        if return_x_mean:
            K, W, S, X_mean = func(data.values, *args, **kwargs)
            K = self._constructor(K, index=data.columns)
            W = self._constructor(W)
            S = self._constructor(S, index=data.index)
            return K, W, S, X_mean
        else:
            K, W, S = func(data.values, *args, **kwargs)
            K = self._constructor(K, index=data.columns)
            W = self._constructor(W)
            S = self._constructor(S, index=data.index)
            return K, W, S

    def dict_learning(self, n_components, alpha, *args, **kwargs):
        """
        Call ``sklearn.decomposition.dict_learning`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.dict_learning
        data = self._data
        code, dictionary, errors = func(data.values, n_components, alpha, *args, **kwargs)
        code = self._constructor(code, index=data.index)
        dictionary = self._constructor(dictionary, columns=data.columns)
        return code, dictionary, errors

    def dict_learning_online(self, *args, **kwargs):
        """
        Call ``sklearn.decomposition.dict_learning_online`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.dict_learning_online
        data = self._data
        return_code = kwargs.get('return_code', True)
        if return_code:
            code, dictionary = func(data.values, *args, **kwargs)
            code = self._constructor(code, index=data.index)
            dictionary = self._constructor(dictionary, columns=data.columns)
            return code, dictionary
        else:
            dictionary = func(data.values, *args, **kwargs)
            dictionary = self._constructor(dictionary, columns=data.columns)
            return dictionary

    def sparse_encode(self, dictionary, *args, **kwargs):
        """
        Call ``sklearn.decomposition.sparce_encode`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        func = self._module.sparse_encode
        data = self._data
        code = func(data.values, dictionary, *args, **kwargs)
        code = self._constructor(code, index=data.index)
        return code



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.dummy as dummy

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestDummy(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.dummy.DummyClassifier, dummy.DummyClassifier)
        self.assertIs(df.dummy.DummyRegressor, dummy.DummyRegressor)

    @pytest.mark.parametrize("algo", ['DummyClassifier'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.dummy, algo)(strategy='most_frequent',
                                       random_state=self.random_state)
        mod2 = getattr(dummy, algo)(strategy='most_frequent',
                                    random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        self.assertEqual(df.score(mod1), mod2.score(iris.data, iris.target))



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods, _wrap_data_target_func


class LinearModelMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.linear_model``.
    """

    _module_name = 'sklearn.linear_model'

    def enet_path(self, *args, **kwargs):
        """
        Call ``sklearn.linear_model.enet_path`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.enet_path
        return self._enet_path_wraps(func, *args, **kwargs)

    def _enet_path_wraps(self, func, *args, **kwargs):
        data = self._data
        target = self._target
        return_models = kwargs.get('return_models', False)
        if return_models:
            models = func(data.values, y=target.values, *args, **kwargs)
            return models
        else:
            alphas, coefs, dual_gaps = func(data.values, y=target.values, *args, **kwargs)
            coefs = self._constructor(coefs, index=data.columns)
            return alphas, coefs, dual_gaps

    def lars_path(self, *args, **kwargs):
        """
        Call ``sklearn.linear_model.lars_path`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.lars_path
        data = self._data
        target = self._target
        alphas, active, coefs = func(data.values, y=target.values, *args, **kwargs)
        coefs = self._constructor(coefs, index=data.columns)
        return alphas, active, coefs

    def lasso_path(self, *args, **kwargs):
        """
        Call ``sklearn.linear_model.lasso_path`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.lasso_path
        # lasso_path internally uses enet_path
        return self._enet_path_wraps(func, *args, **kwargs)

    def lasso_stability_path(self, *args, **kwargs):
        """
        Call ``sklearn.linear_model.lasso_stability_path`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.lasso_stability_path
        data = self._data
        target = self._target
        alpha_grid, scores_path = func(data.values, y=target.values, *args, **kwargs)
        scores_path = self._constructor(scores_path, index=data.columns)
        return alpha_grid, scores_path

    def orthogonal_mp_gram(self, *args, **kwargs):
        """
        Call ``sklearn.linear_model.orthogonal_mp_gram`` using automatic mapping.

        - ``Gram``: ``ModelFrame.data.T.dot(ModelFrame.data)``
        - ``Xy``: ``ModelFrame.data.T.dot(ModelFrame.target)``
        """
        func = self._module.orthogonal_mp_gram
        data = self._data.values
        target = self._target.values
        gram = data.T.dot(data)
        Xy = data.T.dot(target)
        coef = func(gram, Xy, *args, **kwargs)
        return coef


_lm_methods = ['orthogonal_mp']
_attach_methods(LinearModelMethods, _wrap_data_target_func, _lm_methods)



#!/usr/bin/env python

import importlib

import pandas as pd


def _get_version(package):
    try:
        importlib.import_module(package)
        return 'Installed'
    except ImportError:
        return 'Not installed'


def info():
    packages = ['sklearn', 'statsmodels', 'seaborn', 'xgboost']
    versions = [_get_version(p) for p in packages]
    df = pd.DataFrame({'Installed': versions}, index=packages)
    return df



#!/usr/bin/env python

import sklearn.datasets as datasets

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestDatasets(tm.TestCase):

    def test_boston(self):
        data = datasets.load_boston()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (506, 14))

    def test_diabetes(self):
        data = datasets.load_diabetes()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (442, 11))

    def test_digits(self):
        data = datasets.load_digits()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (1797, 65))

    def test_iris(self):
        data = datasets.load_iris()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (150, 5))

    def test_linnerud(self):
        return
        # data must be 1-dimensional

        # data = datasets.load_linnerud()
        # df = pdml.ModelFrame(data)
        # self.assertEqual(df.shape, (150, 5))
